@@ -36,7 +36,7 @@
 #include "ario-debug.h"
 #include "lib/ario-conf.h"
 #include "ario-util.h"
-#include "lib/rb-glade-helpers.h"
+#include "lib/gtk-builder-helpers.h"
 #include "preferences/ario-preferences.h"
 #include "ario-plugin.h"
 #include "servers/ario-server.h"
@@ -102,6 +102,7 @@ typedef struct
         gchar *title;
         guint length;
         time_t play_time;
+        gchar *encoded_play_time;
 } AudioscrobblerEntry;
 
 typedef struct
@@ -241,12 +242,13 @@ ario_audioscrobbler_constructor (GType type,
                                  guint n_construct_properties,
                                  GObjectConstructParam *construct_properties)
 {
+        ARIO_LOG_FUNCTION_START
         GObject *obj;
         ArioAudioscrobbler *audioscrobbler;
 
         /* Invoke parent constructor. */
         ArioAudioscrobblerClass *klass;
-        GObjectClass *parent_class;  
+        GObjectClass *parent_class;
         klass = ARIO_AUDIOSCROBBLER_CLASS (g_type_class_peek (ARIO_TYPE_AUDIOSCROBBLER));
         parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
         obj = parent_class->constructor (type,
@@ -262,6 +264,7 @@ ario_audioscrobbler_constructor (GType type,
 static void
 ario_audioscrobbler_class_init (ArioAudioscrobblerClass *klass)
 {
+        ARIO_LOG_FUNCTION_START
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 
@@ -275,6 +278,7 @@ ario_audioscrobbler_class_init (ArioAudioscrobblerClass *klass)
 static void
 ario_audioscrobbler_init (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         ARIO_LOG_DBG ("Initialising Audioscrobbler");
         ARIO_LOG_DBG ("Plugin ID: %s, Version %s (Protocol %s)",
                       CLIENT_ID, CLIENT_VERSION, SCROBBLER_VERSION);
@@ -308,6 +312,7 @@ ario_audioscrobbler_init (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_dispose (GObject *object)
 {
+        ARIO_LOG_FUNCTION_START
         ArioAudioscrobbler *audioscrobbler;
 
         g_return_if_fail (object != NULL);
@@ -343,6 +348,7 @@ ario_audioscrobbler_dispose (GObject *object)
 static void
 ario_audioscrobbler_finalize (GObject *object)
 {
+        ARIO_LOG_FUNCTION_START
         ArioAudioscrobbler *audioscrobbler;
 
         ARIO_LOG_DBG ("Finalizing Audioscrobbler");
@@ -376,6 +382,7 @@ ario_audioscrobbler_finalize (GObject *object)
 ArioAudioscrobbler*
 ario_audioscrobbler_new (void)
 {
+        ARIO_LOG_FUNCTION_START
         ArioAudioscrobbler *audioscrobbler;
         audioscrobbler = g_object_new (ARIO_TYPE_AUDIOSCROBBLER, NULL);
 
@@ -388,10 +395,11 @@ ario_audioscrobbler_new (void)
 
 #if defined(HAVE_LIBSOUP_2_4)
 static SoupURI *
-ario_audioscrobbler_get_libsoup_uri ()
+ario_audioscrobbler_get_libsoup_uri (void)
 {
+        ARIO_LOG_FUNCTION_START
         SoupURI *uri = NULL;
-        gchar *host;
+        const gchar *host;
 
         if (!ario_conf_get_boolean (PREF_USE_PROXY, PREF_USE_PROXY_DEFAULT))
                 return NULL;
@@ -400,15 +408,15 @@ ario_audioscrobbler_get_libsoup_uri ()
         soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTP);
         host = ario_conf_get_string (PREF_PROXY_ADDRESS, PREF_PROXY_ADDRESS);
         soup_uri_set_host (uri, host);
-        g_free (host);
         soup_uri_set_port (uri, ario_conf_get_integer (PREF_PROXY_PORT, PREF_PROXY_PORT_DEFAULT));
 
         return uri;
 }
 #elif defined(HAVE_LIBSOUP_2_2)
 static SoupUri *
-ario_audioscrobbler_get_libsoup_uri ()
+ario_audioscrobbler_get_libsoup_uri (void)
 {
+        ARIO_LOG_FUNCTION_START
         SoupUri *uri = NULL;
 
         if (!ario_conf_get_boolean (PREF_USE_PROXY, PREF_USE_PROXY_DEFAULT))
@@ -417,7 +425,7 @@ ario_audioscrobbler_get_libsoup_uri ()
         uri = g_new0 (SoupUri, 1);
         uri->protocol = SOUP_PROTOCOL_HTTP;
 
-        uri->host = ario_conf_get_string (PREF_PROXY_ADDRESS, PREF_PROXY_ADDRESS);
+        uri->host = g_strdup (ario_conf_get_string (PREF_PROXY_ADDRESS, PREF_PROXY_ADDRESS));
         uri->port = ario_conf_get_integer (PREF_PROXY_PORT, PREF_PROXY_PORT_DEFAULT);
 
         return uri;
@@ -428,9 +436,10 @@ ario_audioscrobbler_get_libsoup_uri ()
 static void
 ario_audioscrobbler_add_timeout (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         if (!audioscrobbler->priv->timeout_id) {
                 ARIO_LOG_DBG ("Adding Audioscrobbler timer (15 seconds)");
-                audioscrobbler->priv->timeout_id = 
+                audioscrobbler->priv->timeout_id =
                         g_timeout_add (15000, (GSourceFunc) ario_audioscrobbler_timeout_cb,
                                        audioscrobbler);
         }
@@ -439,11 +448,12 @@ ario_audioscrobbler_add_timeout (ArioAudioscrobbler *audioscrobbler)
 static gboolean
 ario_audioscrobbler_is_queueable (ArioServerSong *song)
 {
+        ARIO_LOG_FUNCTION_START
         const char *title;
         const char *artist;
         gulong duration;
 
-        /* TODO: check if the entry is appropriate for sending to 
+        /* TODO: check if the entry is appropriate for sending to
          * audioscrobbler
          */
 
@@ -474,6 +484,7 @@ ario_audioscrobbler_is_queueable (ArioServerSong *song)
 static AudioscrobblerEntry *
 ario_audioscrobbler_create_entry (ArioServerSong *song)
 {
+        ARIO_LOG_FUNCTION_START
         AudioscrobblerEntry *as_entry = g_new0 (AudioscrobblerEntry, 1);
 
         if (song->title) {
@@ -499,7 +510,8 @@ ario_audioscrobbler_create_entry (ArioServerSong *song)
 static gboolean
 ario_audioscrobbler_add_to_queue (ArioAudioscrobbler *audioscrobbler,
                                   AudioscrobblerEntry *entry)
-{        
+{
+        ARIO_LOG_FUNCTION_START
         if (g_queue_get_length (audioscrobbler->priv->queue) < MAX_QUEUE_SIZE){
                 g_queue_push_tail (audioscrobbler->priv->queue, entry);
                 audioscrobbler->priv->queue_changed = TRUE;
@@ -517,6 +529,7 @@ ario_audioscrobbler_add_to_queue (ArioAudioscrobbler *audioscrobbler,
 static void
 maybe_add_current_song_to_queue (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         guint elapsed;
         AudioscrobblerEntry *cur_entry;
 
@@ -553,6 +566,7 @@ maybe_add_current_song_to_queue (ArioAudioscrobbler *audioscrobbler)
 static gboolean
 ario_audioscrobbler_timeout_cb (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         maybe_add_current_song_to_queue (audioscrobbler);
 
         /* do handshake if we need to */
@@ -565,7 +579,7 @@ ario_audioscrobbler_timeout_cb (ArioAudioscrobbler *audioscrobbler)
                 else
                         ario_audioscrobbler_save_queue (audioscrobbler);
         } else {
-                ARIO_LOG_DBG ("the queue is empty"); 
+                ARIO_LOG_DBG ("the queue is empty");
         }
         return TRUE;
 }
@@ -573,6 +587,7 @@ ario_audioscrobbler_timeout_cb (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_parse_response (ArioAudioscrobbler *audioscrobbler, SoupMessage *msg)
 {
+        ARIO_LOG_FUNCTION_START
         gboolean successful;
         ARIO_LOG_DBG ("Parsing response, status=%d", msg->status_code);
 
@@ -681,6 +696,7 @@ ario_audioscrobbler_parse_response (ArioAudioscrobbler *audioscrobbler, SoupMess
 static gboolean
 idle_unref_cb (GObject *object)
 {
+        ARIO_LOG_FUNCTION_START
         g_object_unref (object);
         return FALSE;
 }
@@ -695,6 +711,7 @@ ario_audioscrobbler_perform (ArioAudioscrobbler *audioscrobbler,
                              char *post_data,
                              SoupSessionCallback response_handler)
 {
+        ARIO_LOG_FUNCTION_START
         SoupMessage *msg;
 
         msg = soup_message_new (post_data == NULL ? "GET" : "POST", url);
@@ -729,6 +746,7 @@ ario_audioscrobbler_perform (ArioAudioscrobbler *audioscrobbler,
 static gboolean
 ario_audioscrobbler_should_handshake (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         /* Perform handshake if necessary. Only perform handshake if
          *   - we have no current handshake; AND
          *   - we have waited the appropriate amount of time between
@@ -758,6 +776,7 @@ ario_audioscrobbler_should_handshake (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_do_handshake (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gchar *username;
         gchar *url;
 
@@ -799,6 +818,7 @@ static void
 ario_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
 #endif
 {
+        ARIO_LOG_FUNCTION_START
         ArioAudioscrobbler *audioscrobbler = ARIO_AUDIOSCROBBLER(user_data);
 
         ARIO_LOG_DBG ("Handshake response");
@@ -824,6 +844,7 @@ ario_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
 static gchar *
 ario_audioscrobbler_build_authentication_data (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gchar *md5_password;
         gchar *md5_temp;
         gchar *md5_response;
@@ -836,13 +857,13 @@ ario_audioscrobbler_build_authentication_data (ArioAudioscrobbler *audioscrobble
          *   - Must have md5_challenge
          *   - Queue must not be empty
          */
-        if ((audioscrobbler->priv->username == NULL) 
+        if ((audioscrobbler->priv->username == NULL)
             || (*audioscrobbler->priv->username == '\0')) {
                 ARIO_LOG_DBG ("No username set");
                 return NULL;
         }
 
-        if ((audioscrobbler->priv->password == NULL) 
+        if ((audioscrobbler->priv->password == NULL)
             || (*audioscrobbler->priv->password == '\0')) {
                 ARIO_LOG_DBG ("No password set");
                 return NULL;
@@ -871,7 +892,7 @@ ario_audioscrobbler_build_authentication_data (ArioAudioscrobbler *audioscrobble
                                 NULL);
         md5_response = ario_util_md5 (md5_temp);
 
-        username = soup_uri_encode (audioscrobbler->priv->username, 
+        username = soup_uri_encode (audioscrobbler->priv->username,
                                     EXTRA_URI_ENCODE_CHARS);
         post_data = g_strdup_printf ("u=%s&s=%s&", username, md5_response);
 
@@ -887,6 +908,7 @@ static gchar *
 ario_audioscrobbler_build_post_data (ArioAudioscrobbler *audioscrobbler,
                                      const gchar *authentication_data)
 {
+        ARIO_LOG_FUNCTION_START
         g_return_val_if_fail (!g_queue_is_empty (audioscrobbler->priv->queue),
                               NULL);
 
@@ -912,7 +934,7 @@ ario_audioscrobbler_build_post_data (ArioAudioscrobbler *audioscrobbler,
                 post_data = new;
 
                 /* add to submission list */
-                g_queue_push_tail (audioscrobbler->priv->submission, 
+                g_queue_push_tail (audioscrobbler->priv->submission,
                                    entry);
                 i++;
         } while ((!g_queue_is_empty(audioscrobbler->priv->queue)) && (i < MAX_SUBMIT_SIZE));
@@ -923,6 +945,7 @@ ario_audioscrobbler_build_post_data (ArioAudioscrobbler *audioscrobbler,
 static void
 ario_audioscrobbler_submit_queue (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gchar *auth_data;
 
         auth_data = ario_audioscrobbler_build_authentication_data (audioscrobbler);
@@ -946,6 +969,7 @@ ario_audioscrobbler_submit_queue (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_g_queue_concat (GQueue *q1, GQueue *q2)
 {
+        ARIO_LOG_FUNCTION_START
         GList *elem;
 
         while (!g_queue_is_empty (q2)) {
@@ -985,6 +1009,7 @@ ario_g_queue_concat (GQueue *q1, GQueue *q2)
 static char *
 eel_strdup_strftime (const char *format, struct tm *time_pieces)
 {
+        ARIO_LOG_FUNCTION_START
         GString *string;
         const char *remainder, *percent;
         char code[4], buffer[512];
@@ -1117,10 +1142,21 @@ eel_strdup_strftime (const char *format, struct tm *time_pieces)
         return result;
 }
 
+#ifdef _WIN32
+inline struct tm* localtime_r (const time_t *clock, struct tm *result)
+{
+        ARIO_LOG_FUNCTION_START
+        if (!clock || !result) return NULL;
+        memcpy(result,localtime(clock),sizeof(*result));
+        return result;
+}
+#endif
+
 /* Based on evolution/mail/message-list.c:filter_date() */
 static char *
 ario_utf_friendly_time (time_t date)
 {
+        ARIO_LOG_FUNCTION_START
         time_t nowdate;
         time_t yesdate;
         struct tm then, now, yesterday;
@@ -1210,6 +1246,7 @@ static void
 ario_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 #endif
 {
+        ARIO_LOG_FUNCTION_START
         ArioAudioscrobbler *audioscrobbler = ARIO_AUDIOSCROBBLER (user_data);
 
         ARIO_LOG_DBG ("Submission response");
@@ -1230,7 +1267,7 @@ ario_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
                 ++audioscrobbler->priv->failures;
 
                 /* add failed submission entries back to queue */
-                ario_g_queue_concat (audioscrobbler->priv->submission, 
+                ario_g_queue_concat (audioscrobbler->priv->submission,
                                      audioscrobbler->priv->queue);
                 g_assert (g_queue_is_empty (audioscrobbler->priv->queue));
                 g_queue_free (audioscrobbler->priv->queue);
@@ -1261,11 +1298,12 @@ ario_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 static void
 ario_audioscrobbler_import_settings (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         /* import conf settings. */
         g_free (audioscrobbler->priv->username);
         g_free (audioscrobbler->priv->password);
-        audioscrobbler->priv->username = ario_conf_get_string (PREF_AUDIOSCROBBLER_USERNAME, PREF_AUDIOSCROBBLER_USERNAME_DEFAULT);
-        audioscrobbler->priv->password = ario_conf_get_string (PREF_AUDIOSCROBBLER_PASSWORD, PREF_AUDIOSCROBBLER_PASSWORD_DEFAULT);
+        audioscrobbler->priv->username = g_strdup (ario_conf_get_string (PREF_AUDIOSCROBBLER_USERNAME, PREF_AUDIOSCROBBLER_USERNAME_DEFAULT));
+        audioscrobbler->priv->password = g_strdup (ario_conf_get_string (PREF_AUDIOSCROBBLER_PASSWORD, PREF_AUDIOSCROBBLER_PASSWORD_DEFAULT));
 
         ario_audioscrobbler_add_timeout (audioscrobbler);
         audioscrobbler->priv->status = HANDSHAKING;
@@ -1276,6 +1314,7 @@ ario_audioscrobbler_import_settings (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_preferences_sync (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         const char *status;
         char *free_this = NULL;
         char *v;
@@ -1351,6 +1390,7 @@ ario_audioscrobbler_preferences_sync (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_preferences_response_cb (GtkWidget *dialog, gint response, ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gtk_widget_destroy (audioscrobbler->priv->preferences);
         audioscrobbler->priv->preferences = NULL;
 }
@@ -1359,6 +1399,7 @@ ario_audioscrobbler_preferences_response_cb (GtkWidget *dialog, gint response, A
 static void
 ario_audioscrobbler_preferences_close_cb (GtkWidget *dialog, ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gtk_widget_destroy (audioscrobbler->priv->preferences);
         audioscrobbler->priv->preferences = NULL;
 }
@@ -1367,7 +1408,8 @@ GtkWidget *
 ario_audioscrobbler_get_config_widget (ArioAudioscrobbler *audioscrobbler,
                                        ArioPlugin *plugin)
 {
-        GladeXML *xml;
+        ARIO_LOG_FUNCTION_START
+        GtkBuilder *builder;
         GtkWidget *config_widget;
         gchar *file;
 
@@ -1386,26 +1428,26 @@ ario_audioscrobbler_get_config_widget (ArioAudioscrobbler *audioscrobbler,
                                   G_CALLBACK (ario_audioscrobbler_preferences_close_cb),
                                   audioscrobbler);
 
-                file = ario_plugin_find_file ("audioscrobbler-prefs.glade");
+                file = ario_plugin_find_file ("audioscrobbler-prefs.ui");
                 if (file) {
-                        xml = rb_glade_xml_new (file, "audioscrobbler_vbox", audioscrobbler);
+                        builder = gtk_builder_helpers_new (file, audioscrobbler);
 
-                        config_widget = glade_xml_get_widget (xml, "audioscrobbler_vbox");
-                        audioscrobbler->priv->username_entry = glade_xml_get_widget (xml, "username_entry");
-                        audioscrobbler->priv->username_label = glade_xml_get_widget (xml, "username_label");
-                        audioscrobbler->priv->password_entry = glade_xml_get_widget (xml, "password_entry");
-                        audioscrobbler->priv->password_label = glade_xml_get_widget (xml, "password_label");
-                        audioscrobbler->priv->status_label = glade_xml_get_widget (xml, "status_label");
-                        audioscrobbler->priv->queue_count_label = glade_xml_get_widget (xml, "queue_count_label");
-                        audioscrobbler->priv->submit_count_label = glade_xml_get_widget (xml, "submit_count_label");
-                        audioscrobbler->priv->submit_time_label = glade_xml_get_widget (xml, "submit_time_label");
+                        config_widget = GTK_WIDGET (gtk_builder_get_object (builder, "audioscrobbler_vbox"));
+                        audioscrobbler->priv->username_entry = GTK_WIDGET (gtk_builder_get_object (builder, "username_entry"));
+                        audioscrobbler->priv->username_label = GTK_WIDGET (gtk_builder_get_object (builder, "username_label"));
+                        audioscrobbler->priv->password_entry = GTK_WIDGET (gtk_builder_get_object (builder, "password_entry"));
+                        audioscrobbler->priv->password_label = GTK_WIDGET (gtk_builder_get_object (builder, "password_label"));
+                        audioscrobbler->priv->status_label = GTK_WIDGET (gtk_builder_get_object (builder, "status_label"));
+                        audioscrobbler->priv->queue_count_label = GTK_WIDGET (gtk_builder_get_object (builder, "queue_count_label"));
+                        audioscrobbler->priv->submit_count_label = GTK_WIDGET (gtk_builder_get_object (builder, "submit_count_label"));
+                        audioscrobbler->priv->submit_time_label = GTK_WIDGET (gtk_builder_get_object (builder, "submit_time_label"));
 
-                        rb_glade_boldify_label (xml, "audioscrobbler_label");
-
-                        g_object_unref (xml);
+                        gtk_builder_helpers_boldify_label (builder, "audioscrobbler_label");
 
                         gtk_container_add (GTK_CONTAINER (GTK_DIALOG (audioscrobbler->priv->preferences)->vbox), config_widget);
                         g_free (file);
+
+                        g_object_unref (builder);
                 }
         }
 
@@ -1420,7 +1462,7 @@ static void
 ario_audioscrobbler_conf_username_changed_cb (guint cnxn_id,
                                               ArioAudioscrobbler *audioscrobbler)
 {
-
+        ARIO_LOG_FUNCTION_START
         const gchar *username;
 
         g_free (audioscrobbler->priv->username);
@@ -1444,6 +1486,7 @@ static void
 ario_audioscrobbler_conf_password_changed_cb (guint cnxn_id,
                                               ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         const gchar *password;
 
         g_free (audioscrobbler->priv->password);
@@ -1465,6 +1508,7 @@ static void
 ario_audioscrobbler_song_changed_cb (ArioServer *server,
                                      ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         ArioServerSong *song;
 
         song = ario_server_get_current_song ();
@@ -1497,6 +1541,7 @@ void
 ario_audioscrobbler_username_entry_changed_cb (GtkEntry *entry,
                                                ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         ario_conf_set_string (PREF_AUDIOSCROBBLER_USERNAME,
                               gtk_entry_get_text (entry));
 }
@@ -1505,6 +1550,7 @@ void
 ario_audioscrobbler_username_entry_activate_cb (GtkEntry *entry,
                                                 ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         gtk_widget_grab_focus (audioscrobbler->priv->password_entry);
 }
 
@@ -1512,6 +1558,7 @@ void
 ario_audioscrobbler_password_entry_changed_cb (GtkEntry *entry,
                                                ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         ario_conf_set_string (PREF_AUDIOSCROBBLER_PASSWORD,
                               gtk_entry_get_text (entry));
 }
@@ -1520,6 +1567,7 @@ void
 ario_audioscrobbler_password_entry_activate_cb (GtkEntry *entry,
                                                 ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         /* ? */
 }
 
@@ -1527,19 +1575,23 @@ ario_audioscrobbler_password_entry_activate_cb (GtkEntry *entry,
 static void
 audioscrobbler_entry_init (AudioscrobblerEntry *entry)
 {
+        ARIO_LOG_FUNCTION_START
         entry->artist = g_strdup ("");
         entry->album = g_strdup ("");
         entry->title = g_strdup ("");
         entry->length = 0;
         entry->play_time = 0;
+        entry->encoded_play_time = NULL;
 }
 
 static void
 audioscrobbler_entry_free (AudioscrobblerEntry *entry)
 {
+        ARIO_LOG_FUNCTION_START
         g_free (entry->artist);
         g_free (entry->album);
         g_free (entry->title);
+        g_free (entry->encoded_play_time);
 
         g_free (entry);
 }
@@ -1547,30 +1599,35 @@ audioscrobbler_entry_free (AudioscrobblerEntry *entry)
 static AudioscrobblerEncodedEntry *
 audioscrobbler_entry_encode (AudioscrobblerEntry *entry)
 {
-
+        ARIO_LOG_FUNCTION_START
         AudioscrobblerEncodedEntry *encoded;
 
         encoded = g_new0 (AudioscrobblerEncodedEntry, 1);
 
-        encoded->artist = soup_uri_encode (entry->artist, 
+        encoded->artist = soup_uri_encode (entry->artist,
                                            EXTRA_URI_ENCODE_CHARS);
         encoded->title = soup_uri_encode (entry->title,
                                           EXTRA_URI_ENCODE_CHARS);
-        encoded->album = soup_uri_encode (entry->album, 
+        encoded->album = soup_uri_encode (entry->album,
                                           EXTRA_URI_ENCODE_CHARS);
 
         encoded->timestamp = g_new0 (gchar, 30);
-        strftime (encoded->timestamp, 30, SCROBBLER_DATE_FORMAT, 
-                  gmtime (&entry->play_time));
+        if (entry->encoded_play_time) {
+                encoded->timestamp = g_strndup (entry->encoded_play_time, 30);
+        } else {
+                strftime (encoded->timestamp, 30, SCROBBLER_DATE_FORMAT,
+                                gmtime (&entry->play_time));
+        }
 
         encoded->length = entry->length;
 
         return encoded;
 }
 
-static 
+static
 void audioscrobbler_encoded_entry_free (AudioscrobblerEncodedEntry *entry)
 {
+        ARIO_LOG_FUNCTION_START
         g_free (entry->artist);
         g_free (entry->album);
         g_free (entry->title);
@@ -1585,6 +1642,7 @@ void audioscrobbler_encoded_entry_free (AudioscrobblerEncodedEntry *entry)
 static AudioscrobblerEntry*
 ario_audioscrobbler_load_entry_from_string (const char *string)
 {
+        ARIO_LOG_FUNCTION_START
         AudioscrobblerEntry *entry;
         int i = 0;
         char **breaks;
@@ -1618,10 +1676,7 @@ ario_audioscrobbler_load_entry_from_string (const char *string)
                                 entry->length = atoi (breaks2[1]);
                         }
                         if (g_str_has_prefix (breaks2[0], "i")) {
-                                struct tm tm;
-                                strptime (breaks2[1], SCROBBLER_DATE_FORMAT, 
-                                          &tm);
-                                entry->play_time = mktime (&tm);
+                                entry->encoded_play_time = g_strdup (breaks2[1]);
                         }
                 }
 
@@ -1641,6 +1696,7 @@ ario_audioscrobbler_load_entry_from_string (const char *string)
 static gboolean
 ario_audioscrobbler_load_queue (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         char *pathname;
         gboolean result;
         char *data;
@@ -1686,12 +1742,13 @@ ario_audioscrobbler_load_queue (ArioAudioscrobbler *audioscrobbler)
 static char *
 ario_audioscrobbler_save_entry_to_string (AudioscrobblerEntry *entry)
 {
+        ARIO_LOG_FUNCTION_START
         char *result;
         AudioscrobblerEncodedEntry *encoded;
 
         encoded = audioscrobbler_entry_encode (entry);
         result = g_strdup_printf ("a=%s&t=%s&b=%s&m=&l=%d&i=%s\n",
-                                  encoded->artist, 
+                                  encoded->artist,
                                   encoded->title,
                                   encoded->album,
                                   encoded->length,
@@ -1703,6 +1760,7 @@ ario_audioscrobbler_save_entry_to_string (AudioscrobblerEntry *entry)
 static gboolean
 ario_audioscrobbler_save_queue (ArioAudioscrobbler *audioscrobbler)
 {
+        ARIO_LOG_FUNCTION_START
         char *pathname;
         gboolean ret;
         GString *string = g_string_new (NULL);
@@ -1740,17 +1798,18 @@ ario_audioscrobbler_save_queue (ArioAudioscrobbler *audioscrobbler)
 static void
 ario_audioscrobbler_print_queue (ArioAudioscrobbler *audioscrobbler, gboolean submission)
 {
+        ARIO_LOG_FUNCTION_START
         GList *l;
         AudioscrobblerEntry *entry;
 
         if (submission) {
                 l = audioscrobbler->priv->submission->head;
-                ARIO_LOG_DBG ("Audioscrobbler submission (%d entries): ", 
+                ARIO_LOG_DBG ("Audioscrobbler submission (%d entries): ",
                               g_queue_get_length (audioscrobbler->priv->submission));
 
         } else {
                 l = audioscrobbler->priv->queue->head;
-                ARIO_LOG_DBG ("Audioscrobbler queue (%d entries): ", 
+                ARIO_LOG_DBG ("Audioscrobbler queue (%d entries): ",
                               g_queue_get_length (audioscrobbler->priv->queue));
         }
 
@@ -1762,15 +1821,20 @@ ario_audioscrobbler_print_queue (ArioAudioscrobbler *audioscrobbler, gboolean su
                 ARIO_LOG_DBG ("      album: %s", entry->album);
                 ARIO_LOG_DBG ("      title: %s", entry->title);
                 ARIO_LOG_DBG ("     length: %d", entry->length);
-                strftime (timestamp, 30, SCROBBLER_DATE_FORMAT, 
-                          gmtime (&entry->play_time));
-                ARIO_LOG_DBG ("  timestamp: %s", timestamp);
+                if (entry->encoded_play_time) {
+                        ARIO_LOG_DBG ("  timestamp: %s", entry->encoded_play_time);
+                } else {
+                        strftime (timestamp, 30, SCROBBLER_DATE_FORMAT,
+                                        gmtime (&entry->play_time));
+                        ARIO_LOG_DBG ("  timestamp: %s", timestamp);
+                }
         }
 }
 
 static void
 ario_audioscrobbler_free_queue_entries (ArioAudioscrobbler *audioscrobbler, GQueue **queue)
 {
+        ARIO_LOG_FUNCTION_START
         g_queue_foreach (*queue, (GFunc) audioscrobbler_entry_free, NULL);
         g_queue_free (*queue);
         *queue = NULL;
