@@ -23,15 +23,16 @@
 #include <config.h>
 #include <libxml/parser.h>
 #include <glib/gi18n.h>
-#include "lib/ario-conf.h"
-#include "widgets/ario-songlist.h"
-#include "shell/ario-shell-songinfos.h"
-#include "ario-util.h"
-#include "ario-debug.h"
-#include "servers/ario-server.h"
-#include "preferences/ario-preferences.h"
 
-#define DRAG_THRESHOLD 1
+#include "ario-debug.h"
+#include "ario-util.h"
+#include "lib/ario-conf.h"
+#include "preferences/ario-preferences.h"
+#include "servers/ario-server.h"
+#include "shell/ario-shell-songinfos.h"
+#include "widgets/ario-dnd-tree.h"
+#include "widgets/ario-songlist.h"
+
 #define ROOT "/"
 
 static void ario_filesystem_shutdown (ArioSource *source);
@@ -53,16 +54,8 @@ static void ario_filesystem_cmd_add_play_filesystem (GtkAction *action,
                                                      ArioFilesystem *filesystem);
 static void ario_filesystem_cmd_clear_add_play_filesystem (GtkAction *action,
                                                            ArioFilesystem *filesystem);
-static void ario_filesystem_popup_menu (ArioFilesystem *filesystem);
-static gboolean ario_filesystem_button_press_cb (GtkWidget *widget,
-                                                 GdkEventButton *event,
-                                                 ArioFilesystem *filesystem);
-static gboolean ario_filesystem_button_release_cb (GtkWidget *widget,
-                                                   GdkEventButton *event,
-                                                   ArioFilesystem *filesystem);
-static gboolean ario_filesystem_motion_notify (GtkWidget *widget,
-                                               GdkEventMotion *event,
-                                               ArioFilesystem *filesystem);
+static void ario_filesystem_popup_menu_cb (ArioDndTree* tree,
+                                           ArioFilesystem *filesystem);
 static void ario_filesystem_filetree_drag_data_get_cb (GtkWidget * widget,
                                                        GdkDragContext * context,
                                                        GtkSelectionData * selection_data,
@@ -81,24 +74,21 @@ static void ario_filesystem_fill_filesystem (ArioFilesystem *filesystem);
 
 struct ArioFilesystemPrivate
 {
-        GtkWidget *filesystem;
-        GtkWidget *paned;
-        GtkTreeStore *filesystem_model;
-        GtkTreeSelection *filesystem_selection;
+        GtkWidget *tree;
+        GtkTreeStore *model;
+        GtkTreeSelection *selection;
 
+        GtkWidget *paned;
         GtkWidget *songs;
 
         gboolean connected;
-
-        gboolean dragging;
-        gboolean pressed;
-        gint drag_start_x;
-        gint drag_start_y;
+        gboolean empty;
 
         GtkUIManager *ui_manager;
         GtkActionGroup *actiongroup;
 };
 
+/* Actions on directories */
 static GtkActionEntry ario_filesystem_actions [] =
 {
         { "FilesystemAddDir", GTK_STOCK_ADD, N_("_Add to playlist"), NULL,
@@ -113,6 +103,7 @@ static GtkActionEntry ario_filesystem_actions [] =
 };
 static guint ario_filesystem_n_actions = G_N_ELEMENTS (ario_filesystem_actions);
 
+/* Actions on songs */
 static GtkActionEntry ario_filesystem_songs_actions [] =
 {
         { "FilesystemAddSongs", GTK_STOCK_ADD, N_("_Add to playlist"), NULL,
@@ -130,12 +121,14 @@ static GtkActionEntry ario_filesystem_songs_actions [] =
 };
 static guint ario_filesystem_songs_n_actions = G_N_ELEMENTS (ario_filesystem_songs_actions);
 
+/* Object properties */
 enum
 {
         PROP_0,
         PROP_UI_MANAGER
 };
 
+/* Directory tree columns */
 enum
 {
         FILETREE_ICON_COLUMN,
@@ -145,6 +138,7 @@ enum
         FILETREE_N_COLUMN
 };
 
+/* Drag and drop targets */
 static const GtkTargetEntry dirs_targets  [] = {
         { "text/directory", 0, 0 },
 };
@@ -170,21 +164,35 @@ ario_filesystem_get_icon (ArioSource *source)
         return GTK_STOCK_HARDDISK;
 }
 
+static void
+ario_filesystem_select (ArioSource *source)
+{
+        ArioFilesystem *filesystem = ARIO_FILESYSTEM (source);
+
+        /* Fill directory tree on the first time filesystem tab is selected */
+        if (filesystem->priv->empty)
+                ario_filesystem_fill_filesystem (filesystem);
+}
+
 static void
 ario_filesystem_class_init (ArioFilesystemClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
         ArioSourceClass *source_class = ARIO_SOURCE_CLASS (klass);
 
+        /* GObject virtual methods */
         object_class->set_property = ario_filesystem_set_property;
         object_class->get_property = ario_filesystem_get_property;
 
+        /* ArioSource virtual methods */
         source_class->get_id = ario_filesystem_get_id;
         source_class->get_name = ario_filesystem_get_name;
         source_class->get_icon = ario_filesystem_get_icon;
         source_class->shutdown = ario_filesystem_shutdown;
+        source_class->select = ario_filesystem_select;
 
+        /* Object properties */
         g_object_class_install_property (object_class,
                                          PROP_UI_MANAGER,
                                          g_param_spec_object ("ui-manager",
@@ -193,13 +201,14 @@ ario_filesystem_class_init (ArioFilesystemClass *klass)
                                                               GTK_TYPE_UI_MANAGER,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+        /* Private attributes */
         g_type_class_add_private (klass, sizeof (ArioFilesystemPrivate));
 }
 
 static void
 ario_filesystem_init (ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeViewColumn *column;
         GtkCellRenderer *renderer;
         int pos;
@@ -208,75 +217,74 @@ ario_filesystem_init (ArioFilesystem *filesystem)
         filesystem->priv = ARIO_FILESYSTEM_GET_PRIVATE (filesystem);
 
         filesystem->priv->connected = FALSE;
+        filesystem->priv->empty = TRUE;
 
-        /* Filesystem tree */
+        /* Create scrolled window */
         scrolledwindow_filesystem = gtk_scrolled_window_new (NULL, NULL);
         gtk_widget_show (scrolledwindow_filesystem);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_filesystem), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow_filesystem), GTK_SHADOW_IN);
-        filesystem->priv->filesystem = gtk_tree_view_new ();
-        gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (filesystem->priv->filesystem), TRUE);
 
+        /* Create drag and drop tree */
+        filesystem->priv->tree = ario_dnd_tree_new (dirs_targets,
+                                                    G_N_ELEMENTS (dirs_targets),
+                                                    TRUE);
+        gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (filesystem->priv->tree), TRUE);
+
+        /* Add folder icon column */
         renderer = gtk_cell_renderer_pixbuf_new ();
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_pack_start (column, renderer, FALSE);
         gtk_tree_view_column_set_attributes (column, renderer, "icon-name", FILETREE_ICON_COLUMN, "stock-size", FILETREE_ICONSIZE_COLUMN, NULL);
 
+        /* Add directory name column */
         renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
         gtk_tree_view_column_set_attributes (column, renderer, "text", FILETREE_NAME_COLUMN, NULL);
         gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
         gtk_tree_view_column_set_fixed_width (column, 50);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (filesystem->priv->filesystem), column);
-        filesystem->priv->filesystem_model = gtk_tree_store_new (FILETREE_N_COLUMN,
-                                                                 G_TYPE_STRING,
-                                                                 G_TYPE_UINT,
-                                                                 G_TYPE_STRING,
-                                                                 G_TYPE_STRING);
-        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (filesystem->priv->filesystem_model),
+        gtk_tree_view_append_column (GTK_TREE_VIEW (filesystem->priv->tree), column);
+
+        /* Create tree model */
+        filesystem->priv->model = gtk_tree_store_new (FILETREE_N_COLUMN,
+                                                      G_TYPE_STRING,
+                                                      G_TYPE_UINT,
+                                                      G_TYPE_STRING,
+                                                      G_TYPE_STRING);
+        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (filesystem->priv->model),
                                               0, GTK_SORT_ASCENDING);
-        gtk_tree_view_set_model (GTK_TREE_VIEW (filesystem->priv->filesystem),
-                                 GTK_TREE_MODEL (filesystem->priv->filesystem_model));
-        filesystem->priv->filesystem_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (filesystem->priv->filesystem));
-        gtk_tree_selection_set_mode (filesystem->priv->filesystem_selection,
+        gtk_tree_view_set_model (GTK_TREE_VIEW (filesystem->priv->tree),
+                                 GTK_TREE_MODEL (filesystem->priv->model));
+
+        /* Get tree selection */
+        filesystem->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (filesystem->priv->tree));
+        gtk_tree_selection_set_mode (filesystem->priv->selection,
                                      GTK_SELECTION_BROWSE);
-        gtk_container_add (GTK_CONTAINER (scrolledwindow_filesystem), filesystem->priv->filesystem);
 
-        gtk_drag_source_set (filesystem->priv->filesystem,
-                             GDK_BUTTON1_MASK,
-                             dirs_targets,
-                             G_N_ELEMENTS (dirs_targets),
-                             GDK_ACTION_COPY);
+        /* Add tree to scrolled window */
+        gtk_container_add (GTK_CONTAINER (scrolledwindow_filesystem), filesystem->priv->tree);
 
-        g_signal_connect (GTK_TREE_VIEW (filesystem->priv->filesystem),
+        /* Connect signals for actions on folder tree */
+        g_signal_connect (filesystem->priv->tree,
                           "drag_data_get",
                           G_CALLBACK (ario_filesystem_filetree_drag_data_get_cb), filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
-                          "button_press_event",
-                          G_CALLBACK (ario_filesystem_button_press_cb),
-                          filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
-                          "button_release_event",
-                          G_CALLBACK (ario_filesystem_button_release_cb),
-                          filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
-                          "motion_notify_event",
-                          G_CALLBACK (ario_filesystem_motion_notify),
-                          filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
+        g_signal_connect (filesystem->priv->tree,
+                          "popup",
+                          G_CALLBACK (ario_filesystem_popup_menu_cb), filesystem);
+        g_signal_connect (filesystem->priv->tree,
                           "row-activated",
                           G_CALLBACK (ario_filesystem_filetree_row_activated_cb),
                           filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
+        g_signal_connect (filesystem->priv->tree,
                           "test-expand-row",
                           G_CALLBACK (ario_filesystem_filetree_row_expanded_cb),
                           filesystem);
-        g_signal_connect (filesystem->priv->filesystem,
+        g_signal_connect (filesystem->priv->tree,
                           "cursor-changed",
                           G_CALLBACK (ario_filesystem_cursor_moved_cb),
                           filesystem);
 
-        /* Hpaned properties */
+        /* Create hpaned */
         filesystem->priv->paned = gtk_hpaned_new ();
         gtk_paned_pack1 (GTK_PANED (filesystem->priv->paned), scrolledwindow_filesystem, FALSE, FALSE);
 
@@ -292,13 +300,15 @@ ario_filesystem_shutdown (ArioSource *source)
 {
         ArioFilesystem *filesystem = ARIO_FILESYSTEM (source);
         int pos;
-        int i;
+        guint i;
 
+        /* Save hpaned position */
         pos = gtk_paned_get_position (GTK_PANED (filesystem->priv->paned));
         if (pos > 0)
                 ario_conf_set_integer (PREF_FILSYSTEM_HPANED_SIZE,
                                        pos);
 
+        /* Unregister actions */
         for (i = 0; i < ario_filesystem_n_actions; ++i) {
                 gtk_action_group_remove_action (filesystem->priv->actiongroup,
                                                 gtk_action_group_get_action (filesystem->priv->actiongroup, ario_filesystem_actions[i].name));
@@ -315,7 +325,7 @@ ario_filesystem_set_property (GObject *object,
                               const GValue *value,
                               GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioFilesystem *filesystem = ARIO_FILESYSTEM (object);
 
         switch (prop_id) {
@@ -334,7 +344,7 @@ ario_filesystem_get_property (GObject *object,
                               GValue *value,
                               GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioFilesystem *filesystem = ARIO_FILESYSTEM (object);
 
         switch (prop_id) {
@@ -351,9 +361,8 @@ GtkWidget *
 ario_filesystem_new (GtkUIManager *mgr,
                      GtkActionGroup *group)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioFilesystem *filesystem;
-        GtkWidget *scrolledwindow_songs;
         ArioServer *server = ario_server_get_instance ();
 
         filesystem = g_object_new (TYPE_ARIO_FILESYSTEM,
@@ -373,18 +382,13 @@ ario_filesystem_new (GtkUIManager *mgr,
                                  G_CALLBACK (ario_filesystem_filesystem_changed_cb),
                                  filesystem, 0);
 
-        /* Songs list */
-        scrolledwindow_songs = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_show (scrolledwindow_songs);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_songs), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow_songs), GTK_SHADOW_IN);
+        /* Create songs list */
         filesystem->priv->songs = ario_songlist_new (mgr,
                                                      "/FilesystemSongsPopup",
                                                      FALSE);
-        gtk_paned_pack2 (GTK_PANED (filesystem->priv->paned), scrolledwindow_songs, TRUE, FALSE);
-
-        gtk_container_add (GTK_CONTAINER (scrolledwindow_songs), filesystem->priv->songs);
+        gtk_paned_pack2 (GTK_PANED (filesystem->priv->paned), filesystem->priv->songs, TRUE, FALSE);
 
+        /* Register actions */
         gtk_action_group_add_actions (group,
                                       ario_filesystem_actions,
                                       ario_filesystem_n_actions, filesystem);
@@ -394,41 +398,46 @@ ario_filesystem_new (GtkUIManager *mgr,
 
         filesystem->priv->connected = ario_server_is_connected ();
 
-        ario_filesystem_fill_filesystem (filesystem);
-
         return GTK_WIDGET (filesystem);
 }
 
 static void
 ario_filesystem_fill_filesystem (ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeIter iter, fake_child;
 
-        gtk_tree_store_clear (filesystem->priv->filesystem_model);
+        /* Empty folder tree */
+        gtk_tree_store_clear (filesystem->priv->model);
 
-        gtk_tree_store_append (filesystem->priv->filesystem_model, &iter, NULL);
-        gtk_tree_store_set (filesystem->priv->filesystem_model, &iter,
+        /* Append Root folder */
+        gtk_tree_store_append (filesystem->priv->model, &iter, NULL);
+        gtk_tree_store_set (filesystem->priv->model, &iter,
                             FILETREE_ICON_COLUMN, GTK_STOCK_HARDDISK,
                             FILETREE_ICONSIZE_COLUMN, 1,
                             FILETREE_NAME_COLUMN, _("Music"),
                             FILETREE_DIR_COLUMN, ROOT, -1);
-        gtk_tree_store_append(GTK_TREE_STORE (filesystem->priv->filesystem_model), &fake_child, &iter);
 
-        gtk_tree_selection_unselect_all (filesystem->priv->filesystem_selection);
-        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (filesystem->priv->filesystem_model), &iter)) {
-                gtk_tree_selection_select_iter (filesystem->priv->filesystem_selection, &iter);
-                ario_filesystem_cursor_moved_cb (GTK_TREE_VIEW (filesystem->priv->filesystem), filesystem);
+        /* Append fake child to allow expand */
+        gtk_tree_store_append(GTK_TREE_STORE (filesystem->priv->model), &fake_child, &iter);
+
+        /* Select first row */
+        gtk_tree_selection_unselect_all (filesystem->priv->selection);
+        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (filesystem->priv->model), &iter)) {
+                gtk_tree_selection_select_iter (filesystem->priv->selection, &iter);
+                ario_filesystem_cursor_moved_cb (GTK_TREE_VIEW (filesystem->priv->tree), filesystem);
         }
+        filesystem->priv->empty = FALSE;
 }
 
 static void
 ario_filesystem_state_changed_cb (ArioServer *server,
                                   ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         if (filesystem->priv->connected != ario_server_is_connected ()) {
                 filesystem->priv->connected = ario_server_is_connected ();
+                /* Fill folder tree */
                 ario_filesystem_fill_filesystem (filesystem);
         }
 }
@@ -437,7 +446,8 @@ static void
 ario_filesystem_filesystem_changed_cb (ArioServer *server,
                                        ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Fill folder tree */
         ario_filesystem_fill_filesystem (filesystem);
 }
 
@@ -447,8 +457,7 @@ ario_filesystem_filetree_row_expanded_cb (GtkTreeView *tree_view,
                                           GtkTreePath *path,
                                           ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
-
+        ARIO_LOG_FUNCTION_START;
         ario_filesystem_cursor_moved_cb (tree_view,
                                          filesystem);
 
@@ -461,11 +470,13 @@ ario_filesystem_filetree_row_activated_cb (GtkTreeView *tree_view,
                                            GtkTreeViewColumn *column,
                                            ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
         if (!gtk_tree_view_row_expanded (tree_view, path)) {
+                /* Expand row */
                 gtk_tree_view_expand_row (tree_view, path, FALSE);
         } else {
+                /* Collapse row */
                 gtk_tree_view_collapse_row (tree_view, path);
         }
 }
@@ -474,11 +485,12 @@ static void
 ario_filesystem_cursor_moved_cb (GtkTreeView *tree_view,
                                  ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeIter iter, child, fake_child, song_iter;
-        GtkTreeModel *model = GTK_TREE_MODEL (filesystem->priv->filesystem_model);
-        GtkListStore *liststore = ario_songlist_get_liststore (ARIO_SONGLIST (filesystem->priv->songs));
-        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (filesystem->priv->songs));
+        GtkTreeModel *model = GTK_TREE_MODEL (filesystem->priv->model);
+        ArioSonglist *songlist = ARIO_SONGLIST (filesystem->priv->songs);
+        GtkListStore *liststore = ario_songlist_get_liststore (songlist);
+        GtkTreeSelection *selection = ario_songlist_get_selection (songlist);
         gchar *dir;
         gchar *title;
         GSList *tmp;
@@ -488,41 +500,63 @@ ario_filesystem_cursor_moved_cb (GtkTreeView *tree_view,
         GtkTreePath *treepath;
         gboolean was_expanded;
 
-        if (!gtk_tree_selection_get_selected (filesystem->priv->filesystem_selection,
+        /* Do nothing if no folder is selected */
+        if (!gtk_tree_selection_get_selected (filesystem->priv->selection,
                                               &model,
                                               &iter))
                 return;
 
-        treepath = gtk_tree_model_get_path (GTK_TREE_MODEL (filesystem->priv->filesystem_model), &iter);
+        /* Get treepath of selected folder */
+        treepath = gtk_tree_model_get_path (GTK_TREE_MODEL (filesystem->priv->model), &iter);
+
+        /* Remember if row was expanded or not */
         was_expanded = gtk_tree_view_row_expanded (tree_view, treepath);
 
-        if (gtk_tree_model_iter_children (GTK_TREE_MODEL (filesystem->priv->filesystem_model),
+        /* Remove all childs */
+        if (gtk_tree_model_iter_children (GTK_TREE_MODEL (filesystem->priv->model),
                                           &child,
                                           &iter)) {
-                while (gtk_tree_store_remove (GTK_TREE_STORE (filesystem->priv->filesystem_model), &child)) { }
+                while (gtk_tree_store_remove (GTK_TREE_STORE (filesystem->priv->model), &child)) { }
         }
+
+        /* Empty songs list */
         gtk_list_store_clear (liststore);
-        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->filesystem_model), &iter, FILETREE_DIR_COLUMN, &dir, -1);
+
+        /* Get path of selected dir */
+        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->model), &iter, FILETREE_DIR_COLUMN, &dir, -1);
         g_return_if_fail (dir);
 
+        /* Get files/directories in path */
         files = ario_server_list_files (dir, FALSE);
+
+        /* For each directory */
         for (tmp = files->directories; tmp; tmp = g_slist_next (tmp)) {
                 path = tmp->data;
-                gtk_tree_store_append (filesystem->priv->filesystem_model, &child, &iter);
+                /* Append directory to folder tree */
+                gtk_tree_store_append (filesystem->priv->model, &child, &iter);
                 if (!strcmp (dir, ROOT)) {
                         display_path = path;
                 } else {
+                        /* Do no display parent hierarchy in tree path */
                         display_path = path + strlen (dir) + 1;
                 }
-                gtk_tree_store_set (filesystem->priv->filesystem_model, &child,
+
+                /* Set tree values */
+                gtk_tree_store_set (filesystem->priv->model, &child,
                                     FILETREE_ICON_COLUMN, GTK_STOCK_DIRECTORY,
                                     FILETREE_ICONSIZE_COLUMN, 1,
                                     FILETREE_NAME_COLUMN, display_path,
                                     FILETREE_DIR_COLUMN, path, -1);
-                gtk_tree_store_append(GTK_TREE_STORE (filesystem->priv->filesystem_model), &fake_child, &child);
+
+                /* Append fake child to allow expand */
+                gtk_tree_store_append(GTK_TREE_STORE (filesystem->priv->model), &fake_child, &child);
         }
+
+        /* For each file */
         for (tmp = files->songs; tmp; tmp = g_slist_next (tmp)) {
                 song = tmp->data;
+
+                /* Append song to songs list */
                 gtk_list_store_append (liststore, &song_iter);
 
                 title = ario_util_format_title (song);
@@ -532,15 +566,16 @@ ario_filesystem_cursor_moved_cb (GtkTreeView *tree_view,
                                     SONGS_ALBUM_COLUMN, song->album,
                                     SONGS_FILENAME_COLUMN, song->file,
                                     -1);
-                g_free (title);
         }
         ario_server_free_file_list (files);
         g_free (dir);
 
+        /* Select first song */
         gtk_tree_selection_unselect_all (selection);
         if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (liststore), &song_iter))
                 gtk_tree_selection_select_iter (selection, &song_iter);
 
+        /* Re-expand row if needed */
         if (was_expanded)
                 gtk_tree_view_expand_row (tree_view, treepath, FALSE);
         gtk_tree_path_free (treepath);
@@ -550,22 +585,25 @@ static void
 ario_filesystem_add_filetree (ArioFilesystem *filesystem,
                               gboolean play)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         gchar *dir;
         GtkTreeIter iter;
-        GtkTreeModel *model = GTK_TREE_MODEL (filesystem->priv->filesystem_model);
+        GtkTreeModel *model = GTK_TREE_MODEL (filesystem->priv->model);
 
-        if (!gtk_tree_selection_get_selected (filesystem->priv->filesystem_selection,
+        /* Do nothing if no folder is selected */
+        if (!gtk_tree_selection_get_selected (filesystem->priv->selection,
                                               &model,
                                               &iter))
                 return;
 
-        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->filesystem_model), &iter,
+        /* Get folder path */
+        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->model), &iter,
                             FILETREE_DIR_COLUMN, &dir, -1);
 
         g_return_if_fail (dir);
 
-        ario_playlist_append_dir (dir, play);
+        /* Append songs in folder to playlist */
+        ario_server_playlist_append_dir (dir, play);
         g_free (dir);
 }
 
@@ -573,7 +611,8 @@ static void
 ario_filesystem_cmd_add_filesystem (GtkAction *action,
                                     ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Append songs in selected folder to playlist */
         ario_filesystem_add_filetree (filesystem, FALSE);
 }
 
@@ -581,7 +620,8 @@ static void
 ario_filesystem_cmd_add_play_filesystem (GtkAction *action,
                                          ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Append songs in selected folder to playlist */
         ario_filesystem_add_filetree (filesystem, TRUE);
 }
 
@@ -589,124 +629,34 @@ static void
 ario_filesystem_cmd_clear_add_play_filesystem (GtkAction *action,
                                                ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Empty playlist */
         ario_server_clear ();
+
+        /* Append songs in selected folder to playlist */
         ario_filesystem_add_filetree (filesystem, TRUE);
 }
 
 static void
-ario_filesystem_popup_menu (ArioFilesystem *filesystem)
+ario_filesystem_popup_menu_cb (ArioDndTree* tree,
+                               ArioFilesystem *filesystem)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkWidget *menu;
 
+        /* Show popup menu */
         menu = gtk_ui_manager_get_widget (filesystem->priv->ui_manager, "/FilesystemPopup");
         gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3,
                         gtk_get_current_event_time ());
 }
 
-static gboolean
-ario_filesystem_button_press_cb (GtkWidget *widget,
-                                 GdkEventButton *event,
-                                 ArioFilesystem *filesystem)
-{
-        ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        int x, y;
-
-        if (!GTK_WIDGET_HAS_FOCUS (widget))
-                gtk_widget_grab_focus (widget);
-
-        if (filesystem->priv->dragging)
-                return FALSE;
-
-        if (event->button == 1 && event->type == GDK_2BUTTON_PRESS)
-                return FALSE;
-
-        if (event->button == 1) {
-                gdk_window_get_pointer (widget->window, &x, &y, &mods);
-                filesystem->priv->drag_start_x = x;
-                filesystem->priv->drag_start_y = y;
-                filesystem->priv->pressed = TRUE;
-
-                return FALSE;
-        }
-
-        if (event->button == 3) {
-                GtkTreePath *path;
-
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-                        if (!gtk_tree_selection_path_is_selected (selection, path)) {
-                                gtk_tree_selection_unselect_all (selection);
-                                gtk_tree_selection_select_path (selection, path);
-                        }
-                        ario_filesystem_popup_menu (filesystem);
-                        gtk_tree_path_free (path);
-                        return FALSE;
-                }
-        }
-
-        return FALSE;
-}
-
-static gboolean
-ario_filesystem_button_release_cb (GtkWidget *widget,
-                                   GdkEventButton *event,
-                                   ArioFilesystem *filesystem)
-{
-        ARIO_LOG_FUNCTION_START
-        if (!filesystem->priv->dragging && !(event->state & GDK_CONTROL_MASK) && !(event->state & GDK_SHIFT_MASK)) {
-                GtkTreePath *path;
-
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-                        gtk_tree_selection_unselect_all (selection);
-                        gtk_tree_selection_select_path (selection, path);
-                        gtk_tree_path_free (path);
-                }
-        }
-
-        filesystem->priv->dragging = FALSE;
-        filesystem->priv->pressed = FALSE;
-
-        return FALSE;
-}
-
-static gboolean
-ario_filesystem_motion_notify (GtkWidget *widget,
-                               GdkEventMotion *event,
-                               ArioFilesystem *filesystem)
-{
-        // desactivated to make the logs more readable
-        // ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        int x, y;
-        int dx, dy;
-
-        if ((filesystem->priv->dragging) || !(filesystem->priv->pressed))
-                return FALSE;
-
-        gdk_window_get_pointer (widget->window, &x, &y, &mods);
-
-        dx = x - filesystem->priv->drag_start_x;
-        dy = y - filesystem->priv->drag_start_y;
-
-        if ((ario_util_abs (dx) > DRAG_THRESHOLD) || (ario_util_abs (dy) > DRAG_THRESHOLD))
-                filesystem->priv->dragging = TRUE;
-
-        return FALSE;
-}
-
 static void
 ario_filesystem_filetree_drag_data_get_cb (GtkWidget * widget,
                                            GdkDragContext * context,
                                            GtkSelectionData * selection_data,
                                            guint info, guint time, gpointer data)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioFilesystem *filesystem;
         GtkTreeIter iter;
         GtkTreeModel *model;
@@ -718,16 +668,19 @@ ario_filesystem_filetree_drag_data_get_cb (GtkWidget * widget,
         g_return_if_fail (widget != NULL);
         g_return_if_fail (selection_data != NULL);
 
-        model = GTK_TREE_MODEL (filesystem->priv->filesystem_model);
+        model = GTK_TREE_MODEL (filesystem->priv->model);
 
-        if (!gtk_tree_selection_get_selected (filesystem->priv->filesystem_selection,
+        /* Do nothing if no folder is selected */
+        if (!gtk_tree_selection_get_selected (filesystem->priv->selection,
                                               &model,
                                               &iter))
                 return;
 
-        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->filesystem_model), &iter,
+        /* Get folder path */
+        gtk_tree_model_get (GTK_TREE_MODEL (filesystem->priv->model), &iter,
                             FILETREE_DIR_COLUMN, &dir, -1);
 
+        /* Set drag data */
         gtk_selection_data_set (selection_data, selection_data->target, 8, dir,
                                 strlen ((const gchar*) dir) * sizeof(guchar));
         g_free (dir);
