@@ -18,21 +18,29 @@
  */
 
 #include "servers/ario-mpd.h"
+#include "config.h"
 #include <gtk/gtk.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <limits.h>
 #include <glib/gi18n.h>
-#include "lib/ario-conf.h"
+
+#include "ario-debug.h"
 #include "ario-profiles.h"
 #include "preferences/ario-preferences.h"
-#include "ario-debug.h"
-#include "config.h"
+#include "lib/ario-conf.h"
+#include "widgets/ario-playlist.h"
 
+/* Number of milliseconds in 1 second */
 #define ONE_SECOND 1000
+
+/* Timeout for retrieve of data on MPD */
 #define NORMAL_TIMEOUT 500
 
+/* Reconnect after 0.5 second in case of error */
+#define RECONNECT_INIT_TIMEOUT 500
+/* Multiply reconnection timeout by 2 after each tentative */
+#define RECONNECT_FACTOR 2
+/* Try to reconnect 5 times */
+#define RECONNECT_TENTATIVES 5
+
 static void ario_mpd_finalize (GObject *object);
 static gboolean ario_mpd_connect_to (ArioMpd *mpd,
                                      gchar *hostname,
@@ -40,8 +48,8 @@ static gboolean ario_mpd_connect_to (ArioMpd *mpd,
                                      float timeout);
 static void ario_mpd_connect (void);
 static void ario_mpd_disconnect (void);
-static void ario_mpd_update_db (void);
-static void ario_mpd_check_errors (void);
+static void ario_mpd_update_db (const gchar *path);
+static gboolean ario_mpd_check_errors (void);
 static gboolean ario_mpd_is_connected (void);
 static GSList * ario_mpd_list_tags (const ArioServerTag tag,
                                     const ArioServerCriteria *criteria);
@@ -84,13 +92,13 @@ static GList * ario_mpd_get_songs_info (GSList *paths);
 static ArioServerFileList * ario_mpd_list_files (const char *path,
                                                  gboolean recursive);
 
+/* Private attributes */
 struct ArioMpdPrivate
 {
         mpd_Status *status;
         mpd_Connection *connection;
         mpd_Stats *stats;
 
-        int use_count;
         guint timeout_id;
 
         gboolean support_empty_tags;
@@ -99,6 +107,7 @@ struct ArioMpdPrivate
         gboolean is_updating;
 
         int elapsed;
+        int reconnect_time;
 };
 
 #define ARIO_MPD_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_ARIO_MPD, ArioMpdPrivate))
@@ -110,12 +119,14 @@ static ArioServer *server_instance = NULL;
 static void
 ario_mpd_class_init (ArioMpdClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
         ArioServerInterfaceClass *server_class = ARIO_SERVER_INTERFACE_CLASS (klass);
 
+        /* GObject virtual methods */
         object_class->finalize = ario_mpd_finalize;
 
+        /* ArioServerInterface virtual methods */
         server_class->connect = ario_mpd_connect;
         server_class->disconnect = ario_mpd_disconnect;
         server_class->is_connected = ario_mpd_is_connected;
@@ -153,23 +164,23 @@ ario_mpd_class_init (ArioMpdClass *klass)
         server_class->get_songs_info = ario_mpd_get_songs_info;
         server_class->list_files = ario_mpd_list_files;
 
+        /* Private attributes */
         g_type_class_add_private (klass, sizeof (ArioMpdPrivate));
 }
 
 static void
 ario_mpd_init (ArioMpd *mpd)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         mpd->priv = ARIO_MPD_GET_PRIVATE (mpd);
 
-        mpd->priv->use_count = 0;
         mpd->priv->timeout_id = 0;
 }
 
 static void
 ario_mpd_finalize (GObject *object)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioMpd *mpd;
 
         g_return_if_fail (object != NULL);
@@ -178,15 +189,18 @@ ario_mpd_finalize (GObject *object)
         mpd = ARIO_MPD (object);
         g_return_if_fail (mpd->priv != NULL);
 
+        /* Close connection to MPD */
         if (mpd->priv->connection)
                 mpd_closeConnection (mpd->priv->connection);
 
+        /* Free a few data */
         if (mpd->priv->status)
                 mpd_freeStatus (mpd->priv->status);
 
         if (mpd->priv->stats)
                 mpd_freeStats (mpd->priv->stats);
 
+        /* Stop retrieving data from MPD */
         if (mpd->priv->timeout_id)
                 g_source_remove (mpd->priv->timeout_id);
 
@@ -198,7 +212,8 @@ ario_mpd_finalize (GObject *object)
 ArioMpd *
 ario_mpd_get_instance (ArioServer *server)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Create instance if needed */
         if (!instance) {
                 instance = g_object_new (TYPE_ARIO_MPD, NULL);
                 g_return_val_if_fail (instance->priv != NULL, NULL);
@@ -210,13 +225,15 @@ ario_mpd_get_instance (ArioServer *server)
 static void
 ario_mpd_check_idle (ArioMpd *mpd)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         mpd->priv->support_idle = FALSE;
 #ifdef ENABLE_MPDIDLE
         char *command;
 
+        /* Get list of supported commands */
         mpd_sendCommandsCommand (mpd->priv->connection);
         while ((command = mpd_getNextCommand (mpd->priv->connection))) {
+                /* Detect if idle command is supported */
                 if (!strcmp (command, "idle"))
                         mpd->priv->support_idle = TRUE;
                 g_free (command);
@@ -227,7 +244,7 @@ ario_mpd_check_idle (ArioMpd *mpd)
 static void
 ario_mpd_launch_timeout (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         instance->priv->timeout_id = g_timeout_add (NORMAL_TIMEOUT,
                                                     (GSourceFunc) ario_mpd_update_status,
                                                     NULL);
@@ -237,8 +254,9 @@ ario_mpd_launch_timeout (void)
 static gboolean
 ario_mpd_update_elapsed (gpointer data)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
+        /* If in idle mode: update elapsed time every seconds when MPD is playing */
         if (ario_server_get_current_state() == MPD_STATUS_STATE_PLAY) {
                 ++instance->priv->elapsed;
                 g_object_set (G_OBJECT (instance), "elapsed", instance->priv->elapsed, NULL);
@@ -251,7 +269,7 @@ ario_mpd_update_elapsed (gpointer data)
 static void
 ario_mpd_launch_idle_timeout (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         instance->priv->timeout_id = g_timeout_add (ONE_SECOND,
                                                     (GSourceFunc) ario_mpd_update_elapsed,
                                                     NULL);
@@ -263,8 +281,9 @@ ario_mpd_idle_cb (mpd_Connection *connection,
                   unsigned flags,
                   void *userdata)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
+        /* Update MPD status */
         /* TODO: Be more selective depending on flags */
         if (flags & IDLE_DATABASE
             || flags & IDLE_PLAYLIST
@@ -273,9 +292,11 @@ ario_mpd_idle_cb (mpd_Connection *connection,
             || flags & IDLE_OPTIONS)
                 ario_mpd_update_status ();
 
+        /* Stored playlists changed, update list */
         if (flags & IDLE_STORED_PLAYLIST)
                 g_signal_emit_by_name (G_OBJECT (server_instance), "storedplaylists_changed");
 
+        /* Diconnected from MPD: check errors */
         if (flags & IDLE_DISCONNECT)
                 ario_mpd_check_errors ();
 
@@ -290,16 +311,16 @@ ario_mpd_connect_to (ArioMpd *mpd,
                      int port,
                      float timeout)
 {
-        ARIO_LOG_FUNCTION_START
-        mpd_Stats *stats;
+        ARIO_LOG_FUNCTION_START;
         gchar *password;
         mpd_Connection *connection;
 
+        /* Connect to MPD */
         connection = mpd_newConnection (hostname, port, timeout);
-
         if (!connection)
                 return FALSE;
 
+        /* Check connection errors */
         if  (connection->error) {
                 ARIO_LOG_ERROR("%s", connection->errorStr);
                 mpd_clearError (connection);
@@ -307,34 +328,30 @@ ario_mpd_connect_to (ArioMpd *mpd,
                 return FALSE;
         }
 
+        /* Send password if one is set in profile */
         password = ario_profiles_get_current (ario_profiles_get ())->password;
         if (password) {
                 mpd_sendPasswordCommand (connection, password);
                 mpd_finishCommand (connection);
         }
 
-        mpd_sendStatsCommand (connection);
-        stats = mpd_getStats (connection);
-        mpd_finishCommand (connection);
-        if (stats == NULL) {
-                mpd_closeConnection (connection);
-                mpd->priv->connection = NULL;
-                return FALSE;
-        }
-
-        mpd_freeStats(stats);
         mpd->priv->connection = connection;
 
+        /* Check if idle is supported by MPD server */
         ario_mpd_check_idle (mpd);
 
         if (instance->priv->support_idle && instance->priv->connection) {
 #ifdef ENABLE_MPDIDLE
+                /* Initialise Idle mode */
                 mpd_glibInit (instance->priv->connection);
                 mpd_startIdle (instance->priv->connection, ario_mpd_idle_cb, NULL);
                 g_idle_add ((GSourceFunc) ario_mpd_update_status, NULL);
+
+                /* Launch timeout to update elapsed time */
                 ario_mpd_launch_idle_timeout ();
 #endif
         } else {
+                /* Launch timeout for data retrieve from MPD */
                 ario_mpd_launch_timeout ();
         }
 
@@ -344,7 +361,7 @@ ario_mpd_connect_to (ArioMpd *mpd,
 static gpointer
 ario_mpd_connect_thread (ArioServer *server)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         gchar *hostname;
         int port;
         float timeout;
@@ -374,39 +391,44 @@ ario_mpd_connect_thread (ArioServer *server)
 static void
 ario_mpd_connect (void)
 {
-        ARIO_LOG_FUNCTION_START
-        GtkWidget *win, *vbox,*label, *bar;
+        ARIO_LOG_FUNCTION_START;
+        GtkWidget *win = NULL, *vbox,*label, *bar;
         GThread* thread;
         GtkWidget *dialog;
+        gboolean is_in_error = (instance->priv->reconnect_time > 0);
 
         thread = g_thread_create ((GThreadFunc) ario_mpd_connect_thread,
                                   instance, TRUE, NULL);
 
-        win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-        gtk_window_set_modal (GTK_WINDOW (win), TRUE);
-        vbox = gtk_vbox_new (FALSE, 0);
-        label = gtk_label_new (_("Connecting to server..."));
-        bar = gtk_progress_bar_new ();
+        if (!is_in_error) {
+                win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+                gtk_window_set_modal (GTK_WINDOW (win), TRUE);
+                vbox = gtk_vbox_new (FALSE, 0);
+                label = gtk_label_new (_("Connecting to server..."));
+                bar = gtk_progress_bar_new ();
 
-        gtk_container_add (GTK_CONTAINER (win), vbox);
-        gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 6);
-        gtk_box_pack_start (GTK_BOX (vbox), bar, FALSE, FALSE, 6);
+                gtk_container_add (GTK_CONTAINER (win), vbox);
+                gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 6);
+                gtk_box_pack_start (GTK_BOX (vbox), bar, FALSE, FALSE, 6);
 
-        gtk_window_set_resizable (GTK_WINDOW (win), FALSE);
-        gtk_window_set_title (GTK_WINDOW (win), "Ario");
-        gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_CENTER);
-        gtk_widget_show_all (win);
+                gtk_window_set_resizable (GTK_WINDOW (win), FALSE);
+                gtk_window_set_title (GTK_WINDOW (win), "Ario");
+                gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_CENTER);
+                gtk_widget_show_all (win);
 
-        while (instance->parent.connecting) {
-                gtk_progress_bar_pulse (GTK_PROGRESS_BAR (bar));
-                while (gtk_events_pending ())
-                        gtk_main_iteration ();
-                g_usleep (200000);
+                while (instance->parent.connecting) {
+                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (bar));
+                        while (gtk_events_pending ())
+                                gtk_main_iteration ();
+                        g_usleep (200000);
+                }
         }
 
         g_thread_join (thread);
 
-        if (!ario_server_is_connected ()) {
+        if (ario_server_is_connected ()) {
+                instance->priv->reconnect_time = 0;
+        } else if (!is_in_error) {
                 dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                                  GTK_MESSAGE_ERROR,
                                                  GTK_BUTTONS_OK,
@@ -416,14 +438,16 @@ ario_mpd_connect (void)
                 g_signal_emit_by_name (G_OBJECT (server_instance), "state_changed");
         }
 
-        gtk_widget_hide (win);
-        gtk_widget_destroy (win);
+        if (win) {
+                gtk_widget_hide (win);
+                gtk_widget_destroy (win);
+        }
 }
 
 static void
 ario_mpd_disconnect (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -442,14 +466,18 @@ ario_mpd_disconnect (void)
 }
 
 static void
-ario_mpd_update_db (void)
+ario_mpd_update_db (const gchar *path)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
 
-        mpd_sendUpdateCommand (instance->priv->connection, "");
+        if (!path)
+                mpd_sendUpdateCommand (instance->priv->connection, "");
+        else
+                mpd_sendUpdateCommand (instance->priv->connection, path);
+
         mpd_finishCommand (instance->priv->connection);
 
         if (instance->priv->support_idle && instance->priv->connection)
@@ -459,17 +487,27 @@ ario_mpd_update_db (void)
 static gboolean
 ario_mpd_try_reconnect (gpointer data)
 {
+        ARIO_LOG_FUNCTION_START;
         ario_server_connect ();
+
+        if (!instance->priv->connection
+            && instance->priv->reconnect_time <= RECONNECT_TENTATIVES) {
+                /* Try to reconnect */
+                ++instance->priv->reconnect_time;
+                g_timeout_add (RECONNECT_INIT_TIMEOUT * instance->priv->reconnect_time * RECONNECT_FACTOR,
+                               ario_mpd_try_reconnect, NULL);
+        }
+
         return FALSE;
 }
 
-static void
+static gboolean
 ario_mpd_check_errors (void)
 {
         // desactivated to make the logs more readable
-        //ARIO_LOG_FUNCTION_START
+        //ARIO_LOG_FUNCTION_START;
         if (!instance->priv->connection)
-                return;
+                return FALSE;
 
         if  (instance->priv->connection->error) {
                 ARIO_LOG_ERROR("%s", instance->priv->connection->errorStr);
@@ -477,15 +515,19 @@ ario_mpd_check_errors (void)
                 ario_server_disconnect ();
 
                 /* Try to reconnect */
-                g_timeout_add (500, ario_mpd_try_reconnect, NULL);
+                instance->priv->reconnect_time = 1;
+                g_timeout_add (RECONNECT_INIT_TIMEOUT * instance->priv->reconnect_time * RECONNECT_FACTOR,
+                               ario_mpd_try_reconnect, NULL);
+                return TRUE;
         }
+        return FALSE;
 }
 
 static gboolean
 ario_mpd_is_connected (void)
 {
         // desactivated to make the logs more readable
-        //ARIO_LOG_FUNCTION_START
+        //ARIO_LOG_FUNCTION_START;
         return (instance->priv->connection != NULL);
 }
 
@@ -493,7 +535,7 @@ static GSList *
 ario_mpd_list_tags (const ArioServerTag tag,
                     const ArioServerCriteria *criteria)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         gchar *value;
         const GSList *tmp;
         GSList *values = NULL;
@@ -534,6 +576,7 @@ static gboolean
 ario_mpd_album_is_present (const GSList *albums,
                            const char *album)
 {
+        ARIO_LOG_FUNCTION_START;
         const GSList *tmp;
         ArioServerAlbum *mpd_album;
 
@@ -549,7 +592,7 @@ ario_mpd_album_is_present (const GSList *albums,
 static GSList *
 ario_mpd_get_albums (const ArioServerCriteria *criteria)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *albums = NULL;
         const GSList *tmp;
         mpd_InfoEntity *entity = NULL;
@@ -614,7 +657,8 @@ ario_mpd_get_albums (const ArioServerCriteria *criteria)
                 }
 
                 if (entity->info.song->file) {
-                        mpd_album->path = g_path_get_dirname (entity->info.song->file);
+                        mpd_album->path = entity->info.song->file;
+                        entity->info.song->file = NULL;
                 } else {
                         mpd_album->path = NULL;
                 }
@@ -642,7 +686,7 @@ static GSList *
 ario_mpd_get_songs (const ArioServerCriteria *criteria,
                     const gboolean exact)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *songs = NULL;
         mpd_InfoEntity *entity = NULL;
         const GSList *tmp;
@@ -696,7 +740,7 @@ ario_mpd_get_songs (const ArioServerCriteria *criteria,
 static GSList *
 ario_mpd_get_songs_from_playlist (char *playlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *songs = NULL;
         mpd_InfoEntity *ent = NULL;
 
@@ -721,7 +765,7 @@ ario_mpd_get_songs_from_playlist (char *playlist)
 static GSList *
 ario_mpd_get_playlists (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *playlists = NULL;
         mpd_InfoEntity *ent = NULL;
 
@@ -748,7 +792,7 @@ ario_mpd_get_playlists (void)
 static GSList *
 ario_mpd_get_playlist_changes (gint64 playlist_id)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *songs = NULL;
         mpd_InfoEntity *entity;
 
@@ -757,7 +801,6 @@ ario_mpd_get_playlist_changes (gint64 playlist_id)
                 return NULL;
 
         mpd_sendPlChangesCommand (instance->priv->connection, (long long) playlist_id);
-
         while ((entity = mpd_getNextInfoEntity (instance->priv->connection))) {
                 if (entity->info.song) {
                         songs = g_slist_append (songs, entity->info.song);
@@ -777,7 +820,7 @@ static gboolean
 ario_mpd_update_status (void)
 {
         // desactivated to make the logs more readable
-        //ARIO_LOG_FUNCTION_START
+        //ARIO_LOG_FUNCTION_START;
 
         if (instance->priv->is_updating)
                 return !instance->priv->support_idle;
@@ -792,9 +835,9 @@ ario_mpd_update_status (void)
                 mpd_sendStatusCommand (instance->priv->connection);
                 instance->priv->status = mpd_getStatus (instance->priv->connection);
 
-                ario_mpd_check_errors ();
-
-                if (instance->priv->status) {
+                if (ario_mpd_check_errors ()) {
+                        ario_server_interface_set_default (ARIO_SERVER_INTERFACE (instance));
+                } else if (instance->priv->status) {
                         if (instance->parent.song_id != instance->priv->status->songid
                             || instance->parent.playlist_id != (gint64) instance->priv->status->playlist)
                                 g_object_set (G_OBJECT (instance), "song_id", instance->priv->status->songid, NULL);
@@ -840,7 +883,7 @@ ario_mpd_update_status (void)
 static ArioServerSong *
 ario_mpd_get_current_song_on_server (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioServerSong *song = NULL;
         mpd_InfoEntity *ent ;
 
@@ -862,38 +905,19 @@ ario_mpd_get_current_song_on_server (void)
 static int
 ario_mpd_get_current_playlist_total_time (void)
 {
-        ARIO_LOG_FUNCTION_START
-        int total_time = 0;
-        ArioServerSong *song;
-        mpd_InfoEntity *ent = NULL;
+        ARIO_LOG_FUNCTION_START;
 
         if (!instance->priv->connection)
                 return 0;
 
-        /*
-         * We go to MPD server for each call to this function but it is not a problem as it is
-         * called only once for each playlist change (by status bar). I may change this implementation
-         * if this function is called more than once for performance reasons.
-         */
-        mpd_sendPlaylistInfoCommand(instance->priv->connection, -1);
-        while ((ent = mpd_getNextInfoEntity (instance->priv->connection))) {
-                song = ent->info.song;
-                if (song->time != MPD_SONG_NO_TIME)
-                        total_time = total_time + song->time;
-                mpd_freeInfoEntity(ent);
-        }
-        mpd_finishCommand (instance->priv->connection);
-
-        if (instance->priv->support_idle && instance->priv->connection)
-                mpd_startIdle (instance->priv->connection, ario_mpd_idle_cb, NULL);
-
-        return total_time;
+        /* Compute it from playlist widget (quite coslty but not often called) */
+        return ario_playlist_get_total_time ();
 }
 
 static unsigned long
 ario_mpd_get_last_update (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return 0;
@@ -917,7 +941,7 @@ ario_mpd_get_last_update (void)
 static void
 ario_mpd_do_next (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -932,7 +956,7 @@ ario_mpd_do_next (void)
 static void
 ario_mpd_do_prev (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -947,7 +971,7 @@ ario_mpd_do_prev (void)
 static void
 ario_mpd_do_play (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -962,7 +986,7 @@ ario_mpd_do_play (void)
 static void
 ario_mpd_do_play_pos (gint id)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -978,7 +1002,7 @@ ario_mpd_do_play_pos (gint id)
 static void
 ario_mpd_do_pause (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -993,7 +1017,7 @@ ario_mpd_do_pause (void)
 static void
 ario_mpd_do_stop (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1008,7 +1032,7 @@ ario_mpd_do_stop (void)
 static void
 ario_mpd_set_current_elapsed (const gint elapsed)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1023,7 +1047,7 @@ ario_mpd_set_current_elapsed (const gint elapsed)
 static void
 ario_mpd_set_current_volume (const gint volume)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1039,7 +1063,7 @@ ario_mpd_set_current_volume (const gint volume)
 static void
 ario_mpd_set_current_random (const gboolean random)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1054,7 +1078,7 @@ ario_mpd_set_current_random (const gboolean random)
 static void
 ario_mpd_set_current_repeat (const gboolean repeat)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1069,7 +1093,7 @@ ario_mpd_set_current_repeat (const gboolean repeat)
 static void
 ario_mpd_set_crossfadetime (const int crossfadetime)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1084,7 +1108,7 @@ ario_mpd_set_crossfadetime (const int crossfadetime)
 static void
 ario_mpd_clear (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1100,7 +1124,7 @@ ario_mpd_clear (void)
 static void
 ario_mpd_shuffle (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return;
@@ -1117,7 +1141,7 @@ ario_mpd_shuffle (void)
 static void
 ario_mpd_queue_commit (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *temp;
         ArioServerQueueAction *queue_action;
 
@@ -1167,7 +1191,7 @@ static void
 ario_mpd_insert_at (const GSList *songs,
                     const gint pos)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         int end, offset = 0;
         const GSList *tmp;
 
@@ -1188,7 +1212,7 @@ ario_mpd_insert_at (const GSList *songs,
 static int
 ario_mpd_save_playlist (const char *name)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         mpd_sendSaveCommand (instance->priv->connection, name);
         mpd_finishCommand (instance->priv->connection);
 
@@ -1203,7 +1227,7 @@ ario_mpd_save_playlist (const char *name)
 static void
 ario_mpd_delete_playlist (const char *name)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         mpd_sendRmCommand (instance->priv->connection, name);
         mpd_finishCommand (instance->priv->connection);
 
@@ -1214,7 +1238,7 @@ ario_mpd_delete_playlist (const char *name)
 static GSList *
 ario_mpd_get_outputs (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *outputs = NULL;
         mpd_OutputEntity *output_ent;
 
@@ -1239,7 +1263,7 @@ static void
 ario_mpd_enable_output (int id,
                         gboolean enabled)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
         if (enabled) {
                 mpd_sendEnableOutputCommand(instance->priv->connection, id);
@@ -1256,7 +1280,7 @@ ario_mpd_enable_output (int id,
 static ArioServerStats *
 ario_mpd_get_stats (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         /* check if there is a connection */
         if (!instance->priv->connection)
                 return NULL;
@@ -1277,7 +1301,7 @@ ario_mpd_get_stats (void)
 static GList *
 ario_mpd_get_songs_info (GSList *paths)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         const gchar *path = NULL;
         GSList *temp;
         GList *songs = NULL;
@@ -1315,7 +1339,7 @@ static ArioServerFileList *
 ario_mpd_list_files (const char *path,
                      gboolean recursive)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         mpd_InfoEntity *entity;
         ArioServerFileList *files = (ArioServerFileList *) g_malloc0 (sizeof (ArioServerFileList));
 
