@@ -19,25 +19,21 @@
 
 #include "servers/ario-server-interface.h"
 #include <gtk/gtk.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <glib/gi18n.h>
-#include "ario-debug.h"
 
-#define NORMAL_TIMEOUT 500
-#define LAZY_TIMEOUT 12000
+#include "ario-debug.h"
+#include "ario-util.h"
 
 static void ario_server_interface_finalize (GObject *object);
 static void ario_server_interface_set_property (GObject *object,
-                                      guint prop_id,
-                                      const GValue *value,
-                                      GParamSpec *pspec);
+                                                guint prop_id,
+                                                const GValue *value,
+                                                GParamSpec *pspec);
 static void ario_server_interface_get_property (GObject *object,
-                                      guint prop_id,
-                                      GValue *value,
-                                      GParamSpec *pspec);
+                                                guint prop_id,
+                                                GValue *value,
+                                                GParamSpec *pspec);
 
+/* Object properties */
 enum
 {
         PROP_0,
@@ -53,6 +49,7 @@ enum
 
 G_DEFINE_TYPE (ArioServerInterface, ario_server_interface, G_TYPE_OBJECT)
 
+/* Dummy methods for default behavior */
 static void
 dummy_void_void (void)
 {
@@ -133,19 +130,20 @@ dummy_pointer_int (gint64 playlist_id)
 static void
 ario_server_interface_class_init (ArioServerInterfaceClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+        /* GObject virtual methods */
         object_class->finalize = ario_server_interface_finalize;
-
         object_class->set_property = ario_server_interface_set_property;
         object_class->get_property = ario_server_interface_get_property;
 
+        /* Default virtual methods for ArioServerInterface */
         klass->connect = dummy_void_void;
         klass->disconnect = dummy_void_void;
         klass->is_connected = dummy_int_void;
         klass->update_status = dummy_int_void;
-        klass->update_db = dummy_void_void;
+        klass->update_db = (void (*) (const char *)) dummy_void_pointer;
         klass->list_tags = (GSList* (*) (const ArioServerTag, const ArioServerCriteria *)) dummy_pointer_tag_pointer;
         klass->get_albums = (GSList* (*) (const ArioServerCriteria *)) dummy_pointer_pointer;
         klass->get_songs = (GSList* (*) (const ArioServerCriteria *, const gboolean)) dummy_pointer_pointer_int;
@@ -178,6 +176,7 @@ ario_server_interface_class_init (ArioServerInterfaceClass *klass)
         klass->get_songs_info = (GList* (*) (GSList *)) dummy_pointer_pointer;
         klass->list_files = (ArioServerFileList* (*) (const char *, const int)) dummy_pointer_pointer_int;
 
+        /* Object properties */
         g_object_class_install_property (object_class,
                                          PROP_SONGID,
                                          g_param_spec_int ("song_id",
@@ -246,7 +245,8 @@ ario_server_interface_class_init (ArioServerInterfaceClass *klass)
 static void
 ario_server_interface_init (ArioServerInterface *server_interface)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Initialization of attributes */
         server_interface->song_id = -1;
         server_interface->playlist_id = -1;
         server_interface->volume = -1;
@@ -255,7 +255,7 @@ ario_server_interface_init (ArioServerInterface *server_interface)
 static void
 ario_server_interface_finalize (GObject *object)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioServerInterface *server_interface;
 
         g_return_if_fail (object != NULL);
@@ -263,6 +263,7 @@ ario_server_interface_finalize (GObject *object)
 
         server_interface = ARIO_SERVER_INTERFACE (object);
 
+        /* Free current song */
         if (server_interface->server_song)
                 ario_server_free_song (server_interface->server_song);
 
@@ -275,7 +276,7 @@ ario_server_interface_set_property (GObject *object,
                                     const GValue *value,
                                     GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioServerInterface *server_interface = ARIO_SERVER_INTERFACE (object);
         int song_id;
         gboolean song_changed = FALSE;
@@ -283,13 +284,14 @@ ario_server_interface_set_property (GObject *object,
         switch (prop_id) {
         case PROP_SONGID:
                 song_id = g_value_get_int (value);
+                /* Detect if song has changed */
                 if (server_interface->song_id != song_id) {
                         server_interface->signals_to_emit |= SERVER_SONG_CHANGED_FLAG;
                         server_interface->song_id = song_id;
                         song_changed = TRUE;
                 }
 
-                /* check if there is a connection */
+                /* Check if there is a connection */
                 if (ario_server_is_connected ()) {
                         ArioServerSong *new_song;
                         ArioServerSong *old_song = server_interface->server_song;
@@ -297,39 +299,35 @@ ario_server_interface_set_property (GObject *object,
                         gboolean artist_changed = FALSE;
                         gboolean album_changed = FALSE;
 
+                        /* Get new song on server */
                         new_song = ario_server_get_current_song_on_server ();
+
+                        /* Detect is state has changed */
                         state_changed = (!old_song || !new_song);
                         if (!state_changed) {
-                                artist_changed = ( (old_song->artist && !new_song->artist)
-                                                   || (!old_song->artist && new_song->artist)
-                                                   || (old_song->artist && new_song->artist && g_utf8_collate (old_song->artist, new_song->artist)) );
-                                if (!artist_changed) {
-                                        album_changed = ( (old_song->album && !new_song->album)
-                                                          || (!old_song->album && new_song->album)
-                                                          || (old_song->album && new_song->album && g_utf8_collate (old_song->album, new_song->album)) );
-                                }
-                                if (!song_changed) {
-                                        song_changed = ( (old_song->name && !new_song->name)
-                                                          || (!old_song->name && new_song->name)
-                                                          || (old_song->name && new_song->name && g_utf8_collate (old_song->name, new_song->name)) );
-                                }
+                                /* Detect if artist has changed */
+                                artist_changed = ario_util_strcmp (old_song->artist, new_song->artist) != 0;
+                                if (!artist_changed)
+                                        album_changed = ario_util_strcmp (old_song->album, new_song->album) != 0;
+                                /* Detect if song has changed */
+                                if (!song_changed)
+                                        song_changed = ario_util_strcmp (old_song->name, new_song->name) != 0;
                         }
 
+                        /* Remember to emit album-changed signal */
                         if (state_changed || artist_changed || album_changed)
                                 server_interface->signals_to_emit |= SERVER_ALBUM_CHANGED_FLAG;
 
+                        /* Remember to emit song-changed signal */
                         if (song_changed)
                                 server_interface->signals_to_emit |= SERVER_SONG_CHANGED_FLAG;
 
+                        /* Change current song */
                         if (server_interface->server_song)
                                 ario_server_free_song (server_interface->server_song);
-
-                        if (new_song) {
-                                server_interface->server_song = new_song;
-                        } else {
-                                server_interface->server_song = NULL;
-                        }
+                        server_interface->server_song = new_song;
                 } else {
+                        /* Not connected: free song */
                         if (server_interface->server_song) {
                                 ario_server_free_song (server_interface->server_song);
                                 server_interface->server_song = NULL;
@@ -337,32 +335,39 @@ ario_server_interface_set_property (GObject *object,
                 }
                 break;
         case PROP_STATE:
+                /* Change value and flag signal to emit */
                 server_interface->state = g_value_get_int (value);
                 server_interface->signals_to_emit |= SERVER_STATE_CHANGED_FLAG;
                 break;
         case PROP_VOLUME:
+                /* Change value and flag signal to emit */
                 server_interface->volume = g_value_get_int (value);
                 server_interface->signals_to_emit |= SERVER_VOLUME_CHANGED_FLAG;
                 break;
         case PROP_ELAPSED:
+                /* Change value and flag signal to emit */
                 server_interface->elapsed = g_value_get_int (value);
                 server_interface->signals_to_emit |= SERVER_ELAPSED_CHANGED_FLAG;
                 break;
         case PROP_PLAYLISTID:
+                /* Change value and flag signal to emit */
                 server_interface->playlist_id = g_value_get_int64 (value);
                 if (!ario_server_is_connected ())
                         server_interface->playlist_length = 0;
                 server_interface->signals_to_emit |= SERVER_PLAYLIST_CHANGED_FLAG;
                 break;
         case PROP_RANDOM:
+                /* Change value and flag signal to emit */
                 server_interface->random = g_value_get_boolean (value);
                 server_interface->signals_to_emit |= SERVER_RANDOM_CHANGED_FLAG;
                 break;
         case PROP_REPEAT:
+                /* Change value and flag signal to emit */
                 server_interface->repeat = g_value_get_boolean (value);
                 server_interface->signals_to_emit |= SERVER_REPEAT_CHANGED_FLAG;
                 break;
         case PROP_UPDATINGDB:
+                /* Change value and flag signal to emit */
                 server_interface->updatingdb = g_value_get_int (value);
                 server_interface->signals_to_emit |= SERVER_UPDATINGDB_CHANGED_FLAG;
                 break;
@@ -372,13 +377,13 @@ ario_server_interface_set_property (GObject *object,
         }
 }
 
-static void 
+static void
 ario_server_interface_get_property (GObject *object,
-                          guint prop_id,
-                          GValue *value,
-                          GParamSpec *pspec)
+                                    guint prop_id,
+                                    GValue *value,
+                                    GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioServerInterface *server_interface = ARIO_SERVER_INTERFACE (object);
 
         switch (prop_id) {
@@ -415,27 +420,35 @@ ario_server_interface_get_property (GObject *object,
 void
 ario_server_interface_set_default (ArioServerInterface *server_interface)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Set default song ID */
         if (server_interface->song_id != 0)
                 g_object_set (G_OBJECT (server_interface), "song_id", 0, NULL);
 
+        /* Set default state */
         if (server_interface->state != MPD_STATUS_STATE_UNKNOWN)
                 g_object_set (G_OBJECT (server_interface), "state", MPD_STATUS_STATE_UNKNOWN, NULL);
 
+        /* Set default volume */
         if (server_interface->volume != MPD_STATUS_NO_VOLUME)
                 g_object_set (G_OBJECT (server_interface), "volume", MPD_STATUS_NO_VOLUME, NULL);
 
+        /* Set default elapsed time */
         if (server_interface->elapsed != 0)
                 g_object_set (G_OBJECT (server_interface), "elapsed", 0, NULL);
 
-        g_object_set (G_OBJECT (server_interface), "playlist_id", -1, NULL);
+        /* Set default playlist ID */
+        g_object_set (G_OBJECT (server_interface), "playlist_id", (gint64) -1, NULL);
 
+        /* Set default random value */
         if (server_interface->random != FALSE)
                 g_object_set (G_OBJECT (server_interface), "random", FALSE, NULL);
 
+        /* Set default repeat value */
         if (server_interface->repeat != FALSE)
                 g_object_set (G_OBJECT (server_interface), "repeat", FALSE, NULL);
 
+        /* Set default updatingdb value */
         if (server_interface->updatingdb != 0)
                 g_object_set (G_OBJECT (server_interface), "updatingdb", 0, NULL);
 }
@@ -444,7 +457,8 @@ void
 ario_server_interface_emit (ArioServerInterface *server_interface,
                             ArioServer *server)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Emit signals depending of flags set in signals_to_emit */
         if (server_interface->signals_to_emit & SERVER_SONG_CHANGED_FLAG)
                 g_signal_emit_by_name (G_OBJECT (server), "song_changed");
         if (server_interface->signals_to_emit & SERVER_ALBUM_CHANGED_FLAG)
