@@ -21,15 +21,14 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
-#include <string.h>
 #include <libxml/parser.h>
-#include <time.h>
 #include <glib/gi18n.h>
-#include "lib/rb-glade-helpers.h"
-#include "widgets/ario-playlist.h"
+
 #include "ario-debug.h"
 #include "ario-util.h"
+#include "lib/gtk-builder-helpers.h"
 #include "servers/ario-server.h"
+#include "widgets/ario-playlist.h"
 
 static gboolean ario_shell_similarartists_window_delete_cb (GtkWidget *window,
                                                             GdkEventAny *event,
@@ -47,25 +46,18 @@ G_MODULE_EXPORT void ario_shell_similarartists_addall_cb (GtkButton *button,
 #define MAX_ARTISTS 10
 #define IMAGE_SIZE 120
 
+/* Private attributes */
 struct ArioShellSimilarartistsPrivate
-{      
+{
+        GtkTreeSelection *selection;
         GtkListStore *liststore;
         GThread *thread;
 
-        GtkTreeSelection *selection;
-
         gboolean closed;
-
         const gchar* artist;
 };
 
-typedef struct
-{
-        guchar *name;
-        guchar *image;
-        guchar *url;
-} ArioSimilarArtist;
-
+/* Tree model columns */
 enum
 {
         IMAGE_COLUMN,
@@ -82,24 +74,26 @@ G_DEFINE_TYPE (ArioShellSimilarartists, ario_shell_similarartists, GTK_TYPE_WIND
 static void
 ario_shell_similarartists_class_init (ArioShellSimilarartistsClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Private attributes */
         g_type_class_add_private (klass, sizeof (ArioShellSimilarartistsPrivate));
 }
 
 static void
 ario_shell_similarartists_init (ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         shell_similarartists->priv = ARIO_SHELL_SIMILARARTISTS_GET_PRIVATE (shell_similarartists);
 
+        /* Connect signal for window deletion */
         g_signal_connect (shell_similarartists,
                           "delete_event",
                           G_CALLBACK (ario_shell_similarartists_window_delete_cb),
                           shell_similarartists);
 
+        /* Set window properties */
         gtk_window_set_title (GTK_WINDOW (shell_similarartists), "Ario");
         gtk_window_set_resizable (GTK_WINDOW (shell_similarartists), TRUE);
-
         gtk_container_set_border_width (GTK_CONTAINER (shell_similarartists), 5);
 }
 
@@ -107,49 +101,50 @@ static GSList *
 ario_shell_similarartists_parse_xml_file (char *xmldata,
                                           int size)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         xmlDocPtr doc;
         xmlNodePtr cur;
         xmlNodePtr cur2;
         GSList *similar_artists = NULL;
         ArioSimilarArtist *similar_artist;
 
+        /* Parse XML file */
         doc = xmlParseMemory (xmldata, size);
-
         if (doc == NULL ) {
                 return NULL;
         }
 
+        /* Get root of document */
         cur = xmlDocGetRootElement(doc);
-
         if (!cur) {
                 xmlFreeDoc (doc);
                 return NULL;
         }
 
-        /* We check that the root node name is "similarartists" */
+        /* Check that the root node name is "similarartists" */
         if (xmlStrcmp (cur->name, (const xmlChar *) "similarartists")) {
                 xmlFreeDoc (doc);
                 return NULL;
         }
 
-        if (!cur) {
-                xmlFreeDoc (doc);
-                return NULL;
-        }
-
         for (cur = cur->xmlChildrenNode; cur; cur = cur->next) {
+                /* For each artist node */
                 if (!xmlStrcmp (cur->name, (const xmlChar *) "artist")) {
+                        /* Create an ArioSimilarArtist */
                         similar_artist = (ArioSimilarArtist *) g_malloc0 (sizeof (ArioSimilarArtist));
                         for (cur2 = cur->xmlChildrenNode; cur2; cur2 = cur2->next) {
                                 if ((!xmlStrcmp (cur2->name, (const xmlChar *) "name"))) {
+                                        /* Fill name */
                                         similar_artist->name = xmlNodeListGetString (doc, cur2->xmlChildrenNode, 1);
                                 } else if ((!xmlStrcmp (cur2->name, (const xmlChar *) "image"))) {
+                                        /* Fill image */
                                         similar_artist->image = xmlNodeListGetString (doc, cur2->xmlChildrenNode, 1);
                                 } else if ((!xmlStrcmp (cur2->name, (const xmlChar *) "url"))) {
+                                        /* Fill URL */
                                         similar_artist->url = xmlNodeListGetString (doc, cur2->xmlChildrenNode, 1);
                                 }
                         }
+                        /* Append ArioSimilarArtist to the list */
                         similar_artists = g_slist_append (similar_artists, similar_artist);
                 }
         }
@@ -158,7 +153,8 @@ ario_shell_similarartists_parse_xml_file (char *xmldata,
 
         return similar_artists;
 }
-static void
+
+void
 ario_shell_similarartists_free_similarartist (ArioSimilarArtist *similar_artist)
 {
         if (similar_artist) {
@@ -169,22 +165,13 @@ ario_shell_similarartists_free_similarartist (ArioSimilarArtist *similar_artist)
         }
 }
 
-static gboolean
-ario_shell_similarartists_refresh (gpointer data)
-{
-        while (gtk_events_pending ())
-                gtk_main_iteration ();
-
-        return FALSE;
-}
-
 static gboolean
 ario_shell_similarartists_get_images_foreach (GtkTreeModel *model,
                                               GtkTreePath *path,
                                               GtkTreeIter *iter,
                                               ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         int size;
         char *data;
         GdkPixbufLoader *loader;
@@ -192,11 +179,13 @@ ario_shell_similarartists_get_images_foreach (GtkTreeModel *model,
         int width, height;
         gchar *image_url;
 
+        /* Get image URL of current row */
         gtk_tree_model_get (model,
                             iter,
                             IMAGEURL_COLUMN, &image_url,
                             -1);
 
+        /* Download image */
         ario_util_download_file (image_url,
                                  NULL, 0, NULL,
                                  &size,
@@ -205,6 +194,8 @@ ario_shell_similarartists_get_images_foreach (GtkTreeModel *model,
 
         if (size == 0 || !data)
                 return FALSE;
+
+        /* Create pixbuf from image data */
         loader = gdk_pixbuf_loader_new ();
         gdk_pixbuf_loader_write (loader,
                                  (const guchar *) data,
@@ -217,9 +208,9 @@ ario_shell_similarartists_get_images_foreach (GtkTreeModel *model,
         if (!pixbuf)
                 return FALSE;
 
+        /* Resize image to IMAGE_SIZE, keeping proportions */
         width = gdk_pixbuf_get_width (pixbuf);
         height = gdk_pixbuf_get_height (pixbuf);
-
         if (width > height) {
                 tmp_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
                                                       IMAGE_SIZE,
@@ -234,21 +225,21 @@ ario_shell_similarartists_get_images_foreach (GtkTreeModel *model,
         g_object_unref (G_OBJECT (pixbuf));
         pixbuf = tmp_pixbuf;
 
+        /* Set pixbuf in current row */
         gtk_list_store_set (shell_similarartists->priv->liststore, iter,
                             IMAGE_COLUMN, pixbuf,
                             -1);
         g_object_unref (G_OBJECT (pixbuf));
 
-        g_idle_add (ario_shell_similarartists_refresh, NULL);
-
         return FALSE;
 }
 
 static gpointer
 ario_shell_similarartists_get_images (ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
+        /* Get images of each row */
         gtk_tree_model_foreach (GTK_TREE_MODEL (shell_similarartists->priv->liststore),
                                 (GtkTreeModelForeachFunc) ario_shell_similarartists_get_images_foreach,
                                 shell_similarartists);
@@ -256,20 +247,24 @@ ario_shell_similarartists_get_images (ArioShellSimilarartists *shell_similararti
         return NULL;
 }
 
-static GSList *
+GSList *
 ario_shell_similarartists_get_similar_artists (const gchar *artist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         char *keyword;
         char *xml_uri;
         int xml_size;
         char *xml_data;
         GSList *similar_artists;
 
+        /* Format artist */
         keyword = ario_util_format_keyword (artist);
+
+        /* Get last.fm uri */
         xml_uri = g_strdup_printf (LASTFM_URI, keyword);
         g_free (keyword);
 
+        /* Download XML file */
         ario_util_download_file (xml_uri,
                                  NULL, 0, NULL,
                                  &xml_size,
@@ -279,6 +274,7 @@ ario_shell_similarartists_get_similar_artists (const gchar *artist)
                 return NULL;
         }
 
+        /* Parse XML file */
         similar_artists = ario_shell_similarartists_parse_xml_file (xml_data,
                                                                     xml_size);
         g_free (xml_data);
@@ -289,7 +285,7 @@ ario_shell_similarartists_get_similar_artists (const gchar *artist)
 static void
 ario_shell_similarartists_get_artists (ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *similar_artists, *tmp;
         ArioSimilarArtist *similar_artist;
         GtkTreeIter iter;
@@ -299,18 +295,25 @@ ario_shell_similarartists_get_artists (ArioShellSimilarartists *shell_similarart
         ArioServerAtomicCriteria atomic_criteria;
         ArioServerCriteria *criteria = NULL;
 
+        /* Get list of similar artists */
         similar_artists = ario_shell_similarartists_get_similar_artists (shell_similarartists->priv->artist);
 
         atomic_criteria.tag = MPD_TAG_ITEM_ARTIST;
         criteria = g_slist_append (criteria, &atomic_criteria);
 
+        /* For each similar artist */
         for (tmp = similar_artists; tmp; tmp = g_slist_next (tmp)) {
+                /* Stop here if needed */
                 if (++i > MAX_ARTISTS || shell_similarartists->priv->closed)
                         break;
+
                 similar_artist = tmp->data;
                 atomic_criteria.value = (gchar *) similar_artist->name;
 
+                /* Get all songs of artist */
                 songs = ario_server_get_songs (criteria, TRUE);
+
+                /* Format number of songs for display */
                 if (songs)
                         songs_txt = g_strdup_printf (_("%d songs"), g_slist_length (songs));
                 else
@@ -318,6 +321,7 @@ ario_shell_similarartists_get_artists (ArioShellSimilarartists *shell_similarart
                 g_slist_foreach (songs, (GFunc) ario_server_free_song, NULL);
                 g_slist_free (songs);
 
+                /* Append row */
                 gtk_list_store_append (shell_similarartists->priv->liststore, &iter);
                 gtk_list_store_set (shell_similarartists->priv->liststore, &iter,
                                     ARTIST_COLUMN, similar_artist->name,
@@ -331,6 +335,7 @@ ario_shell_similarartists_get_artists (ArioShellSimilarartists *shell_similarart
         g_slist_foreach (similar_artists, (GFunc) ario_shell_similarartists_free_similarartist, NULL);
         g_slist_free (similar_artists);
 
+        /* Launch a thread to download artist images */
         shell_similarartists->priv->thread = g_thread_create ((GThreadFunc) ario_shell_similarartists_get_images,
                                                               shell_similarartists,
                                                               TRUE,
@@ -341,12 +346,10 @@ ario_shell_similarartists_get_artists (ArioShellSimilarartists *shell_similarart
 GtkWidget *
 ario_shell_similarartists_new (void)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioShellSimilarartists *shell_similarartists;
-        GladeXML *xml;
+        GtkBuilder *builder;
         GtkWidget *treeview;
-        GtkTreeViewColumn *column;
-        GtkCellRenderer *renderer;
         gchar *artist;
 
         artist = ario_server_get_current_artist ();
@@ -359,63 +362,36 @@ ario_shell_similarartists_new (void)
         g_return_val_if_fail (shell_similarartists->priv != NULL, NULL);
         shell_similarartists->priv->closed = FALSE;
 
-        xml = rb_glade_xml_new (GLADE_PATH "similar-artists.glade",
-                                "vbox",
-                                shell_similarartists);
-        treeview = glade_xml_get_widget (xml, "treeview");
+        /* Build UI using GtkBuilder */
+        builder = gtk_builder_helpers_new (UI_PATH "similar-artists.ui",
+                                           shell_similarartists);
 
-        shell_similarartists->priv->liststore = gtk_list_store_new (N_COLUMN,
-                                                                    GDK_TYPE_PIXBUF,
-                                                                    G_TYPE_STRING,
-                                                                    G_TYPE_STRING,
-                                                                    G_TYPE_STRING,
-                                                                    G_TYPE_STRING);
+        /* Get pointers to various widgets */
+        treeview = GTK_WIDGET (gtk_builder_get_object (builder, "treeview"));
+        shell_similarartists->priv->liststore =
+                GTK_LIST_STORE (gtk_builder_get_object (builder, "liststore"));
 
-        renderer = gtk_cell_renderer_pixbuf_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Image"), 
-                                                           renderer,
-                                                           "pixbuf", 
-                                                           IMAGE_COLUMN, NULL);
-
-        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-        gtk_tree_view_column_set_fixed_width (column, IMAGE_SIZE);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-
-
-        renderer = gtk_cell_renderer_text_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Artist"),
-                                                           renderer,
-                                                           "text", ARTIST_COLUMN,
-                                                           NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-
-        renderer = gtk_cell_renderer_text_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Songs"),
-                                                           renderer,
-                                                           "text", SONGS_COLUMN,
-                                                           NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-
-        gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
-                                 GTK_TREE_MODEL (shell_similarartists->priv->liststore));
+        /* Get tree selection */
         shell_similarartists->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
         gtk_tree_selection_set_mode (shell_similarartists->priv->selection,
                                      GTK_SELECTION_BROWSE);
 
+        /* Set window properties */
         gtk_window_set_resizable (GTK_WINDOW (shell_similarartists), TRUE);
         gtk_window_set_default_size (GTK_WINDOW (shell_similarartists), 350, 500);
         gtk_window_set_position (GTK_WINDOW (shell_similarartists), GTK_WIN_POS_CENTER);
-
-        gtk_container_add (GTK_CONTAINER (shell_similarartists), glade_xml_get_widget (xml, "vbox"));
+        gtk_container_add (GTK_CONTAINER (shell_similarartists), GTK_WIDGET (gtk_builder_get_object (builder, "vbox")));
 
         gtk_widget_show_all (GTK_WIDGET (shell_similarartists));
 
+        /* Refresh UI */
         while (gtk_events_pending ())
                 gtk_main_iteration ();
 
+        /* Get similar artists and fill tree */
         shell_similarartists->priv->artist = artist;
         ario_shell_similarartists_get_artists (shell_similarartists);
-        g_object_unref (G_OBJECT (xml));
+        g_object_unref (builder);
 
         return GTK_WIDGET (shell_similarartists);
 }
@@ -425,11 +401,13 @@ ario_shell_similarartists_window_delete_cb (GtkWidget *window,
                                             GdkEventAny *event,
                                             ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         shell_similarartists->priv->closed = TRUE;
 
+        /* Wait for end of images download thread */
         g_thread_join (shell_similarartists->priv->thread);
 
+        /* Destroy window */
         gtk_widget_hide (GTK_WIDGET (shell_similarartists));
         gtk_widget_destroy (GTK_WIDGET (shell_similarartists));
 
@@ -440,11 +418,13 @@ void
 ario_shell_similarartists_close_cb (GtkButton *button,
                                     ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         shell_similarartists->priv->closed = TRUE;
 
+        /* Wait for end of images download thread */
         g_thread_join (shell_similarartists->priv->thread);
 
+        /* Destroy window */
         gtk_widget_hide (GTK_WIDGET (shell_similarartists));
         gtk_widget_destroy (GTK_WIDGET (shell_similarartists));
 }
@@ -453,16 +433,20 @@ void
 ario_shell_similarartists_lastfm_cb (GtkButton *button,
                                      ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeModel *treemodel;
         GtkTreeIter iter;
         gchar *url;
 
+        /* Get selected row */
         if (gtk_tree_selection_get_selected (shell_similarartists->priv->selection,
                                              &treemodel,
                                              &iter)) {
+                /* Get URL of selected row */
                 gtk_tree_model_get (treemodel, &iter,
                                     URL_COLUMN, &url, -1);
+
+                /* Open last.fm URL in web browser */
                 ario_util_load_uri (url);
                 g_free (url);
         }
@@ -472,21 +456,23 @@ void
 ario_shell_similarartists_add_cb (GtkButton *button,
                                   ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeModel *treemodel;
         GtkTreeIter iter;
         GSList *artists = NULL;
         gchar *artist;
 
+        /* Get selected row */
         if (gtk_tree_selection_get_selected (shell_similarartists->priv->selection,
                                              &treemodel,
                                              &iter)) {
+                /* Get artist of selected row */
                 gtk_tree_model_get (treemodel, &iter,
                                     ARTIST_COLUMN, &artist, -1);
 
+                /* Append artist songs to playlist */
                 artists = g_slist_append (artists, artist);
-
-                ario_playlist_append_artists (artists, FALSE);
+                ario_server_playlist_append_artists (artists, FALSE, -1);
 
                 g_slist_foreach (artists, (GFunc) g_free, NULL);
                 g_slist_free (artists);
@@ -499,14 +485,16 @@ ario_shell_similarartists_addall_foreach (GtkTreeModel *model,
                                           GtkTreeIter *iter,
                                           GSList **artists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         gchar *artist;
 
+        /* Get artist of row */
         gtk_tree_model_get (model,
                             iter,
                             ARTIST_COLUMN, &artist,
                             -1);
 
+        /* Append artist to the list */
         *artists = g_slist_append (*artists, artist);
 
         return FALSE;
@@ -516,34 +504,41 @@ void
 ario_shell_similarartists_addall_cb (GtkButton *button,
                                      ArioShellSimilarartists *shell_similarartists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *artists = NULL;
 
+        /* Get list of all artists */
         gtk_tree_model_foreach (GTK_TREE_MODEL (shell_similarartists->priv->liststore),
                                 (GtkTreeModelForeachFunc) ario_shell_similarartists_addall_foreach,
                                 &artists);
 
-        ario_playlist_append_artists (artists, FALSE);
+        /* Appends songs of all artists to playlist */
+        ario_server_playlist_append_artists (artists, FALSE, -1);
 
         g_slist_foreach (artists, (GFunc) g_free, NULL);
         g_slist_free (artists);
 }
 
 void
-ario_shell_similarartists_add_similar_to_playlist (const gchar *artist)
+ario_shell_similarartists_add_similar_to_playlist (const gchar *artist,
+                                                   const int nb_entries)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSimilarArtist *similar_artist;
         GSList *artists = NULL, *similar_artists, *tmp;
 
+        /* Get list of similar artists */
         similar_artists = ario_shell_similarartists_get_similar_artists (artist);
-        for (tmp = similar_artists; tmp; tmp = g_slist_next (tmp)) {
-                similar_artist = tmp->data;
 
+        /* For each similar artist */
+        for (tmp = similar_artists; tmp; tmp = g_slist_next (tmp)) {
+                /* Build a list of artists names */
+                similar_artist = tmp->data;
                 artists = g_slist_append (artists, similar_artist->name);
         }
 
-        ario_playlist_append_artists (artists, FALSE);
+        /* Append songs of artists to playlist */
+        ario_server_playlist_append_artists (artists, FALSE, nb_entries);
 
         g_slist_foreach (similar_artists, (GFunc) ario_shell_similarartists_free_similarartist, NULL);
         g_slist_free (similar_artists);
