@@ -22,19 +22,19 @@
 #include <string.h>
 #include <config.h>
 #include <glib/gi18n.h>
-#include "lib/ario-conf.h"
-#include "widgets/ario-songlist.h"
-#include "shell/ario-shell-songinfos.h"
-#include "ario-util.h"
+
 #include "ario-debug.h"
-#include "servers/ario-server.h"
+#include "ario-util.h"
+#include "lib/ario-conf.h"
 #include "preferences/ario-preferences.h"
+#include "servers/ario-server.h"
+#include "shell/ario-shell-songinfos.h"
+#include "widgets/ario-dnd-tree.h"
 #include "widgets/ario-playlist.h"
+#include "widgets/ario-songlist.h"
 
 #ifdef ENABLE_STOREDPLAYLISTS
 
-#define DRAG_THRESHOLD 1
-
 static void ario_storedplaylists_shutdown (ArioSource *source);
 static void ario_storedplaylists_set_property (GObject *object,
                                                guint prop_id,
@@ -56,46 +56,33 @@ static void ario_storedplaylists_cmd_clear_add_play_storedplaylists (GtkAction *
                                                                      ArioStoredplaylists *storedplaylists);
 static void ario_storedplaylists_cmd_delete_storedplaylists (GtkAction *action,
                                                              ArioStoredplaylists *storedplaylists);
-static void ario_storedplaylists_popup_menu (ArioStoredplaylists *storedplaylists);
-static gboolean ario_storedplaylists_button_press_cb (GtkWidget *widget,
-                                                      GdkEventButton *event,
-                                                      ArioStoredplaylists *storedplaylists);
-static gboolean ario_storedplaylists_button_release_cb (GtkWidget *widget,
-                                                        GdkEventButton *event,
+static void ario_storedplaylists_popup_menu_cb (ArioDndTree* tree,
+                                                ArioStoredplaylists *storedplaylists);
+static void ario_storedplaylists_playlists_activate_cb (ArioDndTree* tree,
                                                         ArioStoredplaylists *storedplaylists);
-static gboolean ario_storedplaylists_motion_notify (GtkWidget *widget,
-                                                    GdkEventMotion *event,
-                                                    ArioStoredplaylists *storedplaylists);
-static void ario_storedplaylists_playlists_selection_drag_foreach (GtkTreeModel *model,
-                                                                   GtkTreePath *path,
-                                                                   GtkTreeIter *iter,
-                                                                   gpointer userdata);
 static void ario_storedplaylists_playlists_drag_data_get_cb (GtkWidget * widget,
                                                              GdkDragContext * context,
                                                              GtkSelectionData * selection_data,
                                                              guint info, guint time, gpointer data);
 static void ario_storedplaylists_playlists_selection_changed_cb (GtkTreeSelection *selection,
                                                                  ArioStoredplaylists *storedplaylists);
+static void ario_storedplaylists_fill_storedplaylists (ArioStoredplaylists *storedplaylists);
 
 struct ArioStoredplaylistsPrivate
 {
-        GtkWidget *storedplaylists;
-        GtkListStore *storedplaylists_model;
-        GtkTreeSelection *storedplaylists_selection;
+        GtkListStore *model;
+        GtkTreeSelection *selection;
 
         GtkWidget *songs;
         GtkWidget *paned;
 
         gboolean connected;
-
-        gboolean dragging;
-        gboolean pressed;
-        gint drag_start_x;
-        gint drag_start_y;
+        gboolean empty;
 
         GtkUIManager *ui_manager;
 };
 
+/* Actions on playlists list */
 static GtkActionEntry ario_storedplaylists_actions [] =
 {
         { "StoredplaylistsAddPlaylists", GTK_STOCK_ADD, N_("_Add to playlist"), NULL,
@@ -113,6 +100,7 @@ static GtkActionEntry ario_storedplaylists_actions [] =
 };
 static guint ario_storedplaylists_n_actions = G_N_ELEMENTS (ario_storedplaylists_actions);
 
+/* Actions on songs list */
 static GtkActionEntry ario_storedplaylists_songs_actions [] =
 {
         { "StoredplaylistsAddSongs", GTK_STOCK_ADD, N_("_Add to playlist"), NULL,
@@ -130,6 +118,7 @@ static GtkActionEntry ario_storedplaylists_songs_actions [] =
 };
 static guint ario_storedplaylists_n_songs_actions = G_N_ELEMENTS (ario_storedplaylists_songs_actions);
 
+/* Object properties */
 enum
 {
         PROP_0,
@@ -142,6 +131,7 @@ enum
         PLAYLISTS_N_COLUMN
 };
 
+/* Drag and drop targets */
 static const GtkTargetEntry songs_targets  [] = {
         { "text/songs-list", 0, 0 },
 };
@@ -167,21 +157,35 @@ ario_storedplaylists_get_icon (ArioSource *source)
         return GTK_STOCK_INDEX;
 }
 
+static void
+ario_storedplaylists_select (ArioSource *source)
+{
+        ArioStoredplaylists *storedplaylists = ARIO_STOREDPLAYLISTS (source);
+
+        /* Fill playlists list if it's empty */
+        if (storedplaylists->priv->empty)
+                ario_storedplaylists_fill_storedplaylists (storedplaylists);
+}
+
 static void
 ario_storedplaylists_class_init (ArioStoredplaylistsClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
         ArioSourceClass *source_class = ARIO_SOURCE_CLASS (klass);
 
+        /* GObject virtual methods */
         object_class->set_property = ario_storedplaylists_set_property;
         object_class->get_property = ario_storedplaylists_get_property;
 
+        /* ArioSource virtual methods */
         source_class->get_id = ario_storedplaylists_get_id;
         source_class->get_name = ario_storedplaylists_get_name;
         source_class->get_icon = ario_storedplaylists_get_icon;
         source_class->shutdown = ario_storedplaylists_shutdown;
+        source_class->select = ario_storedplaylists_select;
 
+        /* Object properties */
         g_object_class_install_property (object_class,
                                          PROP_UI_MANAGER,
                                          g_param_spec_object ("ui-manager",
@@ -190,28 +194,37 @@ ario_storedplaylists_class_init (ArioStoredplaylistsClass *klass)
                                                               GTK_TYPE_UI_MANAGER,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+        /* Private attributes */
         g_type_class_add_private (klass, sizeof (ArioStoredplaylistsPrivate));
 }
 
 static void
 ario_storedplaylists_init (ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeViewColumn *column;
         GtkCellRenderer *renderer;
-        int pos;
         GtkWidget *scrolledwindow_storedplaylists;
+        GtkWidget *tree;
+        int pos;
 
         storedplaylists->priv = ARIO_STOREDPLAYLISTS_GET_PRIVATE (storedplaylists);
 
         storedplaylists->priv->connected = FALSE;
+        storedplaylists->priv->empty = TRUE;
 
-        /* Storedplaylists list */
+        /* Create scrolled window for playlists list */
         scrolledwindow_storedplaylists = gtk_scrolled_window_new (NULL, NULL);
         gtk_widget_show (scrolledwindow_storedplaylists);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_storedplaylists), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow_storedplaylists), GTK_SHADOW_IN);
-        storedplaylists->priv->storedplaylists = gtk_tree_view_new ();
+
+        /* Drag and drop tree for playlists list */
+        tree = ario_dnd_tree_new (songs_targets,
+                                  G_N_ELEMENTS (songs_targets),
+                                  FALSE);
+
+        /* Add column for playlists list */
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_("Playlist"),
                                                            renderer,
@@ -219,40 +232,34 @@ ario_storedplaylists_init (ArioStoredplaylists *storedplaylists)
                                                            NULL);
         gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
         gtk_tree_view_column_set_fixed_width (column, 50);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (storedplaylists->priv->storedplaylists), column);
-        storedplaylists->priv->storedplaylists_model = gtk_list_store_new (PLAYLISTS_N_COLUMN,
-                                                                           G_TYPE_STRING);
-        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (storedplaylists->priv->storedplaylists_model),
+        gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+
+        /* Create model for playlists list */
+        storedplaylists->priv->model = gtk_list_store_new (PLAYLISTS_N_COLUMN,
+                                                           G_TYPE_STRING);
+        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (storedplaylists->priv->model),
                                               0, GTK_SORT_ASCENDING);
-        gtk_tree_view_set_model (GTK_TREE_VIEW (storedplaylists->priv->storedplaylists),
-                                 GTK_TREE_MODEL (storedplaylists->priv->storedplaylists_model));
-        storedplaylists->priv->storedplaylists_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (storedplaylists->priv->storedplaylists));
-        gtk_tree_selection_set_mode (storedplaylists->priv->storedplaylists_selection,
+        gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
+                                 GTK_TREE_MODEL (storedplaylists->priv->model));
+
+        /* Get selection for playlists list */
+        storedplaylists->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        gtk_tree_selection_set_mode (storedplaylists->priv->selection,
                                      GTK_SELECTION_MULTIPLE);
-        gtk_container_add (GTK_CONTAINER (scrolledwindow_storedplaylists), storedplaylists->priv->storedplaylists);
 
-        gtk_drag_source_set (storedplaylists->priv->storedplaylists,
-                             GDK_BUTTON1_MASK,
-                             songs_targets,
-                             G_N_ELEMENTS (songs_targets),
-                             GDK_ACTION_COPY);
+        gtk_container_add (GTK_CONTAINER (scrolledwindow_storedplaylists), tree);
 
-        g_signal_connect (GTK_TREE_VIEW (storedplaylists->priv->storedplaylists),
+        /* Signals for actions on playlists list */
+        g_signal_connect (GTK_TREE_VIEW (tree),
                           "drag_data_get",
                           G_CALLBACK (ario_storedplaylists_playlists_drag_data_get_cb), storedplaylists);
-        g_signal_connect (storedplaylists->priv->storedplaylists,
-                          "button_press_event",
-                          G_CALLBACK (ario_storedplaylists_button_press_cb),
-                          storedplaylists);
-        g_signal_connect (storedplaylists->priv->storedplaylists,
-                          "button_release_event",
-                          G_CALLBACK (ario_storedplaylists_button_release_cb),
-                          storedplaylists);
-        g_signal_connect (storedplaylists->priv->storedplaylists,
-                          "motion_notify_event",
-                          G_CALLBACK (ario_storedplaylists_motion_notify),
-                          storedplaylists);
-        g_signal_connect (storedplaylists->priv->storedplaylists_selection,
+        g_signal_connect (GTK_TREE_VIEW (tree),
+                          "popup",
+                          G_CALLBACK (ario_storedplaylists_popup_menu_cb), storedplaylists);
+        g_signal_connect (GTK_TREE_VIEW (tree),
+                          "activate",
+                          G_CALLBACK (ario_storedplaylists_playlists_activate_cb), storedplaylists);
+        g_signal_connect (storedplaylists->priv->selection,
                           "changed",
                           G_CALLBACK (ario_storedplaylists_playlists_selection_changed_cb),
                           storedplaylists);
@@ -275,6 +282,7 @@ ario_storedplaylists_shutdown (ArioSource *source)
         ArioStoredplaylists *storedplaylists = ARIO_STOREDPLAYLISTS (source);
         int pos;
 
+        /* Save hpaned position */
         pos = gtk_paned_get_position (GTK_PANED (storedplaylists->priv->paned));
         if (pos > 0)
                 ario_conf_set_integer (PREF_PLAYLISTS_HPANED_SIZE,
@@ -287,7 +295,7 @@ ario_storedplaylists_set_property (GObject *object,
                                    const GValue *value,
                                    GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioStoredplaylists *storedplaylists = ARIO_STOREDPLAYLISTS (object);
 
         switch (prop_id) {
@@ -306,7 +314,7 @@ ario_storedplaylists_get_property (GObject *object,
                                    GValue *value,
                                    GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioStoredplaylists *storedplaylists = ARIO_STOREDPLAYLISTS (object);
 
         switch (prop_id) {
@@ -323,9 +331,8 @@ GtkWidget *
 ario_storedplaylists_new (GtkUIManager *mgr,
                           GtkActionGroup *group)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioStoredplaylists *storedplaylists;
-        GtkWidget *scrolledwindow_songs;
         ArioServer *server = ario_server_get_instance ();
 
         storedplaylists = g_object_new (TYPE_ARIO_STOREDPLAYLISTS,
@@ -336,24 +343,19 @@ ario_storedplaylists_new (GtkUIManager *mgr,
 
         /* Signals to synchronize the storedplaylists with server */
         g_signal_connect_object (server,
-                                 "state_changed", G_CALLBACK (ario_storedplaylists_connectivity_changed_cb),
+                                 "connectivity_changed", G_CALLBACK (ario_storedplaylists_connectivity_changed_cb),
                                  storedplaylists, 0);
         g_signal_connect_object (server,
                                  "storedplaylists_changed", G_CALLBACK (ario_storedplaylists_storedplaylists_changed_cb),
                                  storedplaylists, 0);
 
-        /* Songs list */
-        scrolledwindow_songs = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_show (scrolledwindow_songs);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_songs), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow_songs), GTK_SHADOW_IN);
+        /* Create songs list */
         storedplaylists->priv->songs = ario_songlist_new (mgr,
                                                           "/StoredplaylistsSongsPopup",
                                                           FALSE);
-        gtk_paned_pack2 (GTK_PANED (storedplaylists->priv->paned), scrolledwindow_songs, TRUE, FALSE);
-
-        gtk_container_add (GTK_CONTAINER (scrolledwindow_songs), storedplaylists->priv->songs);
+        gtk_paned_pack2 (GTK_PANED (storedplaylists->priv->paned), storedplaylists->priv->songs, TRUE, FALSE);
 
+        /* Register actions */
         gtk_action_group_add_actions (group,
                                       ario_storedplaylists_actions,
                                       ario_storedplaylists_n_actions, storedplaylists);
@@ -367,29 +369,35 @@ ario_storedplaylists_new (GtkUIManager *mgr,
 static void
 ario_storedplaylists_fill_storedplaylists (ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeIter storedplaylists_iter;
         GSList *playlists;
         GSList *tmp;
 
-        gtk_list_store_clear (storedplaylists->priv->storedplaylists_model);
+        storedplaylists->priv->empty = FALSE;
+
+        /* Empty playlists list */
+        gtk_list_store_clear (storedplaylists->priv->model);
 
         if (!storedplaylists->priv->connected)
                 return;
 
+        /* Get playlists list on server */
         playlists = ario_server_get_playlists ();
         for (tmp = playlists; tmp; tmp = g_slist_next (tmp)) {
-                gtk_list_store_append (storedplaylists->priv->storedplaylists_model, &storedplaylists_iter);
-                gtk_list_store_set (storedplaylists->priv->storedplaylists_model, &storedplaylists_iter, 0,
+                /* Add playlist to list */
+                gtk_list_store_append (storedplaylists->priv->model, &storedplaylists_iter);
+                gtk_list_store_set (storedplaylists->priv->model, &storedplaylists_iter, 0,
                                     tmp->data, -1);
         }
 
-        g_slist_foreach(playlists, (GFunc) g_free, NULL);
+        g_slist_foreach (playlists, (GFunc) g_free, NULL);
         g_slist_free (playlists);
 
-        gtk_tree_selection_unselect_all (storedplaylists->priv->storedplaylists_selection);
-        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (storedplaylists->priv->storedplaylists_model), &storedplaylists_iter))
-                gtk_tree_selection_select_iter (storedplaylists->priv->storedplaylists_selection, &storedplaylists_iter);
+        /* Select first playlist */
+        gtk_tree_selection_unselect_all (storedplaylists->priv->selection);
+        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (storedplaylists->priv->model), &storedplaylists_iter))
+                gtk_tree_selection_select_iter (storedplaylists->priv->selection, &storedplaylists_iter);
 }
 
 static void
@@ -398,14 +406,14 @@ ario_storedplaylists_playlists_selection_foreach (GtkTreeModel *model,
                                                   GtkTreeIter *iter,
                                                   gpointer userdata)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioStoredplaylists *storedplaylists = ARIO_STOREDPLAYLISTS (userdata);
         gchar* playlist = NULL;
         GSList *songs = NULL, *temp;
         ArioServerSong *song;
         GtkTreeIter song_iter;
         gchar *title;
-        GtkListStore *liststore = ario_songlist_get_liststore (ARIO_SONGLIST (storedplaylists->priv->songs));
+        GtkListStore *liststore;
 
         g_return_if_fail (IS_ARIO_STOREDPLAYLISTS (storedplaylists));
 
@@ -414,10 +422,13 @@ ario_storedplaylists_playlists_selection_foreach (GtkTreeModel *model,
         if (!playlist)
                 return;
 
+        /* Get list of songs of selected playlist */
         songs = ario_server_get_songs_from_playlist (playlist);
         g_free (playlist);
 
+        liststore = ario_songlist_get_liststore (ARIO_SONGLIST (storedplaylists->priv->songs));
         for (temp = songs; temp; temp = g_slist_next (temp)) {
+                /* Append each song to the list */
                 song = temp->data;
                 gtk_list_store_append (liststore, &song_iter);
 
@@ -428,7 +439,6 @@ ario_storedplaylists_playlists_selection_foreach (GtkTreeModel *model,
                                     SONGS_ALBUM_COLUMN, song->album,
                                     SONGS_FILENAME_COLUMN, song->file,
                                     -1);
-                g_free (title);
         }
 
         g_slist_foreach (songs, (GFunc) ario_server_free_song, NULL);
@@ -438,17 +448,21 @@ ario_storedplaylists_playlists_selection_foreach (GtkTreeModel *model,
 static void
 ario_storedplaylists_playlists_selection_update (ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkTreeIter song_iter;
-        GtkListStore *liststore = ario_songlist_get_liststore (ARIO_SONGLIST (storedplaylists->priv->songs));
-        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (storedplaylists->priv->songs));
+        ArioSonglist *songlist = ARIO_SONGLIST (storedplaylists->priv->songs);
+        GtkListStore *liststore = ario_songlist_get_liststore (songlist);
+        GtkTreeSelection *selection = ario_songlist_get_selection (songlist);
 
+        /* Empty songs list */
         gtk_list_store_clear (liststore);
 
-        gtk_tree_selection_selected_foreach (storedplaylists->priv->storedplaylists_selection,
+        /* Add songs of each selected playlist */
+        gtk_tree_selection_selected_foreach (storedplaylists->priv->selection,
                                              ario_storedplaylists_playlists_selection_foreach,
                                              storedplaylists);
 
+        /* Select first song */
         gtk_tree_selection_unselect_all (selection);
         if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (liststore), &song_iter))
                 gtk_tree_selection_select_iter (selection, &song_iter);
@@ -458,7 +472,7 @@ static void
 ario_storedplaylists_playlists_selection_changed_cb (GtkTreeSelection *selection,
                                                      ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ario_storedplaylists_playlists_selection_update (storedplaylists);
 }
 
@@ -466,16 +480,20 @@ static void
 ario_storedplaylists_connectivity_changed_cb (ArioServer *server,
                                               ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         storedplaylists->priv->connected = ario_server_is_connected ();
-        ario_storedplaylists_fill_storedplaylists (storedplaylists);
+
+        /* Fill playlists list */
+        if (!storedplaylists->priv->empty)
+                ario_storedplaylists_fill_storedplaylists (storedplaylists);
 }
 
 static void
 ario_storedplaylists_storedplaylists_changed_cb (ArioServer *server,
                                                  ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Fill playlists list */
         ario_storedplaylists_fill_storedplaylists (storedplaylists);
 }
 
@@ -485,10 +503,11 @@ storedplaylists_foreach (GtkTreeModel *model,
                          GtkTreeIter *iter,
                          gpointer userdata)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList **storedplaylists = (GSList **) userdata;
         gchar *val = NULL;
 
+        /* Append playlist name to the list */
         gtk_tree_model_get (model, iter, PLAYLISTS_NAME_COLUMN, &val, -1);
         *storedplaylists = g_slist_append (*storedplaylists, val);
 }
@@ -497,18 +516,22 @@ static void
 ario_storedplaylists_add_playlists (ArioStoredplaylists *storedplaylists,
                                     gboolean play)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *playlists = NULL;
         GSList *songs = NULL;
         GSList *tmp;
 
-        gtk_tree_selection_selected_foreach (storedplaylists->priv->storedplaylists_selection,
+        /* Get a list of playlists names */
+        gtk_tree_selection_selected_foreach (storedplaylists->priv->selection,
                                              storedplaylists_foreach,
                                              &playlists);
 
         for (tmp = playlists; tmp; tmp = g_slist_next (tmp)) {
+                /* Get songs from playlist */
                 songs = ario_server_get_songs_from_playlist (tmp->data);
-                ario_playlist_append_server_songs (songs, play);
+
+                /* Append songs to main playlist */
+                ario_server_playlist_append_server_songs (songs, play);
 
                 g_slist_foreach (songs, (GFunc) ario_server_free_song, NULL);
                 g_slist_free (songs);
@@ -521,7 +544,10 @@ ario_storedplaylists_add_playlists (ArioStoredplaylists *storedplaylists,
 static void
 ario_storedplaylists_clear_add_play_playlists (ArioStoredplaylists *storedplaylists)
 {
+        /* Empty playlist */
         ario_server_clear ();
+
+        /* Add songs to playlist and play */
         ario_storedplaylists_add_playlists (storedplaylists, TRUE);
 }
 
@@ -529,7 +555,8 @@ static void
 ario_storedplaylists_cmd_add_storedplaylists (GtkAction *action,
                                               ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Add songs to playlist */
         ario_storedplaylists_add_playlists (storedplaylists, FALSE);
 }
 
@@ -537,7 +564,8 @@ static void
 ario_storedplaylists_cmd_add_play_storedplaylists (GtkAction *action,
                                                    ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Add songs to playlist and play */
         ario_storedplaylists_add_playlists (storedplaylists, TRUE);
 }
 
@@ -545,7 +573,7 @@ static void
 ario_storedplaylists_cmd_clear_add_play_storedplaylists (GtkAction *action,
                                                          ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ario_storedplaylists_clear_add_play_playlists (storedplaylists);
 }
 
@@ -553,26 +581,31 @@ static void
 ario_storedplaylists_cmd_delete_storedplaylists (GtkAction *action,
                                                  ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkWidget *dialog;
         gint retval = GTK_RESPONSE_NO;
-        GSList *playlists = NULL;        GSList *tmp;
+        GSList *playlists = NULL;
+        GSList *tmp;
 
+        /* Create confirmation dialog for playlist deletion */
         dialog = gtk_message_dialog_new (NULL,
                                          GTK_DIALOG_MODAL,
                                          GTK_MESSAGE_QUESTION,
                                          GTK_BUTTONS_YES_NO,
                                          _("Are you sure that you want to delete all the selected playlists?"));
 
+        /* Run dialog */
         retval = gtk_dialog_run (GTK_DIALOG(dialog));
         gtk_widget_destroy (dialog);
         if (retval != GTK_RESPONSE_YES)
                 return;
 
-        gtk_tree_selection_selected_foreach (storedplaylists->priv->storedplaylists_selection,
+        /* Get list of selected playlists */
+        gtk_tree_selection_selected_foreach (storedplaylists->priv->selection,
                                              storedplaylists_foreach,
                                              &playlists);
 
+        /* Delete each selected playlist */
         for (tmp = playlists; tmp; tmp = g_slist_next (tmp)) {
                 ario_server_delete_playlist (tmp->data);
         }
@@ -582,143 +615,26 @@ ario_storedplaylists_cmd_delete_storedplaylists (GtkAction *action,
 }
 
 static void
-ario_storedplaylists_popup_menu (ArioStoredplaylists *storedplaylists)
-{
-        ARIO_LOG_FUNCTION_START
-        GtkWidget *menu;
-
-        menu = gtk_ui_manager_get_widget (storedplaylists->priv->ui_manager, "/StoredplaylistsPopup");
-        gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3,
-                        gtk_get_current_event_time ());
-}
-
-static gboolean
-ario_storedplaylists_button_press_cb (GtkWidget *widget,
-                                      GdkEventButton *event,
-                                      ArioStoredplaylists *storedplaylists)
-{
-        ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        GtkTreePath *path;
-        int x, y;
-        gboolean selected;
-
-        if (!GTK_WIDGET_HAS_FOCUS (widget))
-                gtk_widget_grab_focus (widget);
-
-        if (storedplaylists->priv->dragging)
-                return FALSE;
-
-        if (event->state & GDK_CONTROL_MASK || event->state & GDK_SHIFT_MASK)
-                return FALSE;
-
-        if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
-                ario_storedplaylists_add_playlists (storedplaylists, FALSE);
-                return FALSE;
-        }
-
-        if (event->button == 1) {
-                gdk_window_get_pointer (widget->window, &x, &y, &mods);
-                storedplaylists->priv->drag_start_x = x;
-                storedplaylists->priv->drag_start_y = y;
-                storedplaylists->priv->pressed = TRUE;
-
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        selected = gtk_tree_selection_path_is_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), path);
-                        gtk_tree_path_free (path);
-
-                        return selected;
-                }
-
-                return TRUE;
-        }
-
-        if (event->button == 3) {
-                GtkTreePath *path;
-
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-                        if (!gtk_tree_selection_path_is_selected (selection, path)) {
-                                gtk_tree_selection_unselect_all (selection);
-                                gtk_tree_selection_select_path (selection, path);
-                        }
-                        ario_storedplaylists_popup_menu (storedplaylists);
-                        gtk_tree_path_free (path);
-                        return TRUE;
-                }
-        }
-
-        return FALSE;
-}
-
-static gboolean
-ario_storedplaylists_button_release_cb (GtkWidget *widget,
-                                        GdkEventButton *event,
-                                        ArioStoredplaylists *storedplaylists)
-{
-        ARIO_LOG_FUNCTION_START
-        if (!storedplaylists->priv->dragging && !(event->state & GDK_CONTROL_MASK) && !(event->state & GDK_SHIFT_MASK)) {
-                GtkTreePath *path;
-
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-
-                        if (gtk_tree_selection_path_is_selected (selection, path)
-                            && (gtk_tree_selection_count_selected_rows (selection) != 1)) {
-                                gtk_tree_selection_unselect_all (selection);
-                                gtk_tree_selection_select_path (selection, path);
-                        }
-                        gtk_tree_path_free (path);
-                }
-        }
-
-        storedplaylists->priv->dragging = FALSE;
-        storedplaylists->priv->pressed = FALSE;
-
-        return FALSE;
-}
-
-static gboolean
-ario_storedplaylists_motion_notify (GtkWidget *widget,
-                                    GdkEventMotion *event,
+ario_storedplaylists_popup_menu_cb (ArioDndTree* tree,
                                     ArioStoredplaylists *storedplaylists)
 {
-        // desactivated to make the logs more readable
-        // ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        int x, y;
-        int dx, dy;
+        ARIO_LOG_FUNCTION_START;
+        GtkWidget *menu;
 
-        if ((storedplaylists->priv->dragging) || !(storedplaylists->priv->pressed))
-                return FALSE;
-
-        gdk_window_get_pointer (widget->window, &x, &y, &mods);
-
-        dx = x - storedplaylists->priv->drag_start_x;
-        dy = y - storedplaylists->priv->drag_start_y;
-
-        if ((ario_util_abs (dx) > DRAG_THRESHOLD) || (ario_util_abs (dy) > DRAG_THRESHOLD))
-                storedplaylists->priv->dragging = TRUE;
-
-        return FALSE;
+        if (gtk_tree_selection_count_selected_rows (storedplaylists->priv->selection) > 0) {
+                /* Show popup menu */
+                menu = gtk_ui_manager_get_widget (storedplaylists->priv->ui_manager, "/StoredplaylistsPopup");
+                gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3,
+                                gtk_get_current_event_time ());
+        }
 }
 
 static void
-ario_storedplaylists_playlists_selection_drag_foreach (GtkTreeModel *model,
-                                                       GtkTreePath *path,
-                                                       GtkTreeIter *iter,
-                                                       gpointer userdata)
+ario_storedplaylists_playlists_activate_cb (ArioDndTree* tree,
+                                            ArioStoredplaylists *storedplaylists)
 {
-        ARIO_LOG_FUNCTION_START
-        GSList **playlists = (GSList **) userdata;
-        gchar *val = NULL;
-
-        gtk_tree_model_get (model, iter, PLAYLISTS_NAME_COLUMN, &val, -1);
-
-        *playlists = g_slist_append (*playlists, val);
+        ARIO_LOG_FUNCTION_START;
+        ario_storedplaylists_add_playlists (storedplaylists, FALSE);
 }
 
 static void
@@ -727,7 +643,7 @@ ario_storedplaylists_playlists_drag_data_get_cb (GtkWidget * widget,
                                                  GtkSelectionData * selection_data,
                                                  guint info, guint time, gpointer data)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioStoredplaylists *storedplaylists;
         ArioServerSong *song;
         GSList *playlists = NULL;
@@ -741,11 +657,12 @@ ario_storedplaylists_playlists_drag_data_get_cb (GtkWidget * widget,
         g_return_if_fail (widget != NULL);
         g_return_if_fail (selection_data != NULL);
 
-
-        gtk_tree_selection_selected_foreach (storedplaylists->priv->storedplaylists_selection,
-                                             ario_storedplaylists_playlists_selection_drag_foreach,
+        /* Get list of selected playlists */
+        gtk_tree_selection_selected_foreach (storedplaylists->priv->selection,
+                                             storedplaylists_foreach,
                                              &playlists);
 
+        /* Get string of playlist names concatenation */
         str_playlists = g_string_new("");
         for (tmp = playlists; tmp; tmp = g_slist_next (tmp)) {
                 songs = ario_server_get_songs_from_playlist (tmp->data);
@@ -763,6 +680,7 @@ ario_storedplaylists_playlists_drag_data_get_cb (GtkWidget * widget,
         g_slist_foreach (playlists, (GFunc) g_free, NULL);
         g_slist_free (playlists);
 
+        /* Set drag data */
         gtk_selection_data_set (selection_data, selection_data->target, 8, (const guchar *) str_playlists->str,
                                 strlen (str_playlists->str) * sizeof(guchar));
 
