@@ -22,12 +22,12 @@
 #include <string.h>
 #include <config.h>
 #include <glib/gi18n.h>
-#include "widgets/ario-playlist.h"
-#include "shell/ario-shell-songinfos.h"
+
 #include "ario-util.h"
 #include "ario-debug.h"
-
-#define DRAG_THRESHOLD 1
+#include "shell/ario-shell-songinfos.h"
+#include "widgets/ario-dnd-tree.h"
+#include "widgets/ario-playlist.h"
 
 static void ario_songlist_finalize (GObject *object);
 static void ario_songlist_set_property (GObject *object,
@@ -40,16 +40,10 @@ static void ario_songlist_get_property (GObject *object,
                                         GParamSpec *pspec);
 static void ario_songlist_add_in_playlist (ArioSonglist *songlist,
                                            gboolean play);
-static void ario_songlist_popup_menu (ArioSonglist *songlist);
-static gboolean ario_songlist_button_press_cb (GtkWidget *widget,
-                                               GdkEventButton *event,
-                                               ArioSonglist *songlist);
-static gboolean ario_songlist_button_release_cb (GtkWidget *widget,
-                                                 GdkEventButton *event,
-                                                 ArioSonglist *songlist);
-static gboolean ario_songlist_motion_notify_cb (GtkWidget *widget,
-                                                GdkEventMotion *event,
-                                                ArioSonglist *songlist);
+static void ario_songlist_popup_menu_cb (ArioDndTree* tree,
+                                         ArioSonglist *songlist);
+static void ario_songlist_activate_cb (ArioDndTree* tree,
+                                       ArioSonglist *songlist);
 static void ario_songlist_songlists_selection_drag_foreach (GtkTreeModel *model,
                                                             GtkTreePath *path,
                                                             GtkTreeIter *iter,
@@ -61,43 +55,42 @@ static void ario_songlist_drag_data_get_cb (GtkWidget * widget,
                                             guint info, guint time, gpointer data);
 struct ArioSonglistPrivate
 {
-        GtkListStore *songlists_model;
-        GtkTreeSelection *songlists_selection;
-
-        gboolean dragging;
-        gboolean pressed;
-        gint drag_start_x;
-        gint drag_start_y;
+        GtkTreeView* tree;
+        GtkListStore *model;
+        GtkTreeSelection *selection;
 
         GtkUIManager *ui_manager;
 
         gchar *popup;
 };
 
+/* Properties */
 enum
 {
         PROP_0,
         PROP_UI_MANAGER
 };
 
+/* Drag and drop targets */
 static const GtkTargetEntry songs_targets  [] = {
         { "text/songs-list", 0, 0 },
 };
 
 #define ARIO_SONGLIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_ARIO_SONGLIST, ArioSonglistPrivate))
-G_DEFINE_TYPE (ArioSonglist, ario_songlist, GTK_TYPE_TREE_VIEW)
+G_DEFINE_TYPE (ArioSonglist, ario_songlist, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
 ario_songlist_class_init (ArioSonglistClass *klass)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+        /* Virtual Methods */
         object_class->finalize = ario_songlist_finalize;
-
         object_class->set_property = ario_songlist_set_property;
         object_class->get_property = ario_songlist_get_property;
 
+        /* Object properties */
         g_object_class_install_property (object_class,
                                          PROP_UI_MANAGER,
                                          g_param_spec_object ("ui-manager",
@@ -106,13 +99,14 @@ ario_songlist_class_init (ArioSonglistClass *klass)
                                                               GTK_TYPE_UI_MANAGER,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+        /* Private attributes */
         g_type_class_add_private (klass, sizeof (ArioSonglistPrivate));
 }
 
 static void
 ario_songlist_init (ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
 
         songlist->priv = ARIO_SONGLIST_GET_PRIVATE (songlist);
 }
@@ -120,7 +114,7 @@ ario_songlist_init (ArioSonglist *songlist)
 static void
 ario_songlist_finalize (GObject *object)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSonglist *songlist;
 
         g_return_if_fail (object != NULL);
@@ -140,7 +134,7 @@ ario_songlist_set_property (GObject *object,
                             const GValue *value,
                             GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSonglist *songlist = ARIO_SONGLIST (object);
 
         switch (prop_id) {
@@ -159,7 +153,7 @@ ario_songlist_get_property (GObject *object,
                             GValue *value,
                             GParamSpec *pspec)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSonglist *songlist = ARIO_SONGLIST (object);
 
         switch (prop_id) {
@@ -177,7 +171,7 @@ ario_songlist_new (GtkUIManager *mgr,
                    gchar *popup,
                    gboolean is_sortable)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSonglist *songlist;
         GtkTreeViewColumn *column;
         GtkCellRenderer *renderer;
@@ -188,7 +182,16 @@ ario_songlist_new (GtkUIManager *mgr,
 
         g_return_val_if_fail (songlist->priv != NULL, NULL);
 
-        /* Titles */
+        /* Scrolled window properties */
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (songlist), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (songlist), GTK_SHADOW_IN);
+
+        /* Create treeview */
+        songlist->priv->tree = GTK_TREE_VIEW (ario_dnd_tree_new (songs_targets,
+                                                                 G_N_ELEMENTS (songs_targets),
+                                                                 FALSE));
+
+        /* Titles column */
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_("Title"),
                                                            renderer,
@@ -201,9 +204,9 @@ ario_songlist_new (GtkUIManager *mgr,
                 gtk_tree_view_column_set_sort_indicator (column, TRUE);
                 gtk_tree_view_column_set_sort_column_id (column, SONGS_TITLE_COLUMN);
         }
-        gtk_tree_view_append_column (GTK_TREE_VIEW (songlist), column);
+        gtk_tree_view_append_column (songlist->priv->tree, column);
 
-        /* Artists */
+        /* Artists column */
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_("Artist"),
                                                            renderer,
@@ -216,9 +219,9 @@ ario_songlist_new (GtkUIManager *mgr,
                 gtk_tree_view_column_set_sort_indicator (column, TRUE);
                 gtk_tree_view_column_set_sort_column_id (column, SONGS_TITLE_COLUMN);
         }
-        gtk_tree_view_append_column (GTK_TREE_VIEW (songlist), column);
+        gtk_tree_view_append_column (songlist->priv->tree, column);
 
-        /* Albums */
+        /* Albums column */
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_("Album"),
                                                            renderer,
@@ -231,45 +234,37 @@ ario_songlist_new (GtkUIManager *mgr,
                 gtk_tree_view_column_set_sort_indicator (column, TRUE);
                 gtk_tree_view_column_set_sort_column_id (column, SONGS_TITLE_COLUMN);
         }
-        gtk_tree_view_append_column (GTK_TREE_VIEW (songlist), column);
+        gtk_tree_view_append_column (songlist->priv->tree, column);
 
-        songlist->priv->songlists_model = gtk_list_store_new (SONGS_N_COLUMN,
-                                                              G_TYPE_STRING,
-                                                              G_TYPE_STRING,
-                                                              G_TYPE_STRING,
-                                                              G_TYPE_STRING);
+        /* Create model */
+        songlist->priv->model = gtk_list_store_new (SONGS_N_COLUMN,
+                                                    G_TYPE_STRING,
+                                                    G_TYPE_STRING,
+                                                    G_TYPE_STRING,
+                                                    G_TYPE_STRING);
+        gtk_tree_view_set_model (songlist->priv->tree,
+                                 GTK_TREE_MODEL (songlist->priv->model));
 
-        gtk_tree_view_set_model (GTK_TREE_VIEW (songlist),
-                                 GTK_TREE_MODEL (songlist->priv->songlists_model));
-        songlist->priv->songlists_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (songlist));
-        gtk_tree_selection_set_mode (songlist->priv->songlists_selection,
+        /* Get selection */
+        songlist->priv->selection = gtk_tree_view_get_selection (songlist->priv->tree);
+        gtk_tree_selection_set_mode (songlist->priv->selection,
                                      GTK_SELECTION_MULTIPLE);
 
-        gtk_drag_source_set (GTK_WIDGET (songlist),
-                             GDK_BUTTON1_MASK,
-                             songs_targets,
-                             G_N_ELEMENTS (songs_targets),
-                             GDK_ACTION_COPY);
-
-        g_signal_connect (GTK_TREE_VIEW (songlist),
+        /* Connect a few signals for interaction with tree view */
+        g_signal_connect (songlist->priv->tree,
                           "drag_data_get",
                           G_CALLBACK (ario_songlist_drag_data_get_cb), songlist);
-
-        g_signal_connect (songlist,
-                          "button_press_event",
-                          G_CALLBACK (ario_songlist_button_press_cb),
-                          songlist);
-        g_signal_connect (songlist,
-                          "button_release_event",
-                          G_CALLBACK (ario_songlist_button_release_cb),
-                          songlist);
-        g_signal_connect (songlist,
-                          "motion_notify_event",
-                          G_CALLBACK (ario_songlist_motion_notify_cb),
-                          songlist);
+        g_signal_connect (GTK_TREE_VIEW (songlist->priv->tree),
+                          "popup",
+                          G_CALLBACK (ario_songlist_popup_menu_cb), songlist);
+        g_signal_connect (GTK_TREE_VIEW (songlist->priv->tree),
+                          "activate",
+                          G_CALLBACK (ario_songlist_activate_cb), songlist);
 
         songlist->priv->popup = g_strdup (popup);
 
+        gtk_container_add (GTK_CONTAINER (songlist), GTK_WIDGET (songlist->priv->tree));
+
         return GTK_WIDGET (songlist);
 }
 
@@ -279,12 +274,12 @@ songlists_foreach (GtkTreeModel *model,
                    GtkTreeIter *iter,
                    gpointer userdata)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList **songlists = (GSList **) userdata;
         gchar *val = NULL;
 
+        /* Append filename to the list */
         gtk_tree_model_get (model, iter, SONGS_FILENAME_COLUMN, &val, -1);
-
         *songlists = g_slist_append (*songlists, val);
 }
 
@@ -292,13 +287,16 @@ static void
 ario_songlist_add_in_playlist (ArioSonglist *songlist,
                                gboolean play)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *songlists = NULL;
 
-        gtk_tree_selection_selected_foreach (songlist->priv->songlists_selection,
+        /* List of selected songs */
+        gtk_tree_selection_selected_foreach (songlist->priv->selection,
                                              songlists_foreach,
                                              &songlists);
-        ario_playlist_append_songs (songlists, play);
+
+        /* Append songs to playlist */
+        ario_server_playlist_append_songs (songlists, play);
 
         g_slist_foreach (songlists, (GFunc) g_free, NULL);
         g_slist_free (songlists);
@@ -308,7 +306,8 @@ void
 ario_songlist_cmd_add_songlists (GtkAction *action,
                                  ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Add songs to playlist */
         ario_songlist_add_in_playlist (songlist, FALSE);
 }
 
@@ -316,7 +315,8 @@ void
 ario_songlist_cmd_add_play_songlists (GtkAction *action,
                                       ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Add songs to playlist and play */
         ario_songlist_add_in_playlist (songlist, TRUE);
 }
 
@@ -324,8 +324,11 @@ void
 ario_songlist_cmd_clear_add_play_songlists (GtkAction *action,
                                             ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
+        /* Cleat playlist */
         ario_server_clear ();
+
+        /* Add songs to playlist and play */
         ario_songlist_add_in_playlist (songlist, TRUE);
 }
 
@@ -333,33 +336,36 @@ void
 ario_songlist_cmd_songs_properties (GtkAction *action,
                                     ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GSList *paths = NULL;
-        GList *songs;
         GtkWidget *songinfos;
 
-        gtk_tree_selection_selected_foreach (songlist->priv->songlists_selection,
+        /* Get list of selected songs */
+        gtk_tree_selection_selected_foreach (songlist->priv->selection,
                                              songlists_foreach,
                                              &paths);
 
-        songs = ario_server_get_songs_info (paths);
-        g_slist_foreach (paths, (GFunc) g_free, NULL);
-        g_slist_free (paths);
-
-        if (songs) {
-                songinfos = ario_shell_songinfos_new (songs);
+        if (paths) {
+                /* Launch songinfos dialog for selected songs */
+                songinfos = ario_shell_songinfos_new (paths);
                 if (songinfos)
                         gtk_widget_show_all (songinfos);
+
+                g_slist_foreach (paths, (GFunc) g_free, NULL);
+                g_slist_free (paths);
         }
 }
 
 static void
-ario_songlist_popup_menu (ArioSonglist *songlist)
+ario_songlist_popup_menu_cb (ArioDndTree* tree,
+                             ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GtkWidget *menu;
 
-        if (songlist->priv->popup) {
+        /* Show popup */
+        if (songlist->priv->popup
+            && gtk_tree_selection_count_selected_rows (songlist->priv->selection) > 0) {
                 menu = gtk_ui_manager_get_widget (songlist->priv->ui_manager, songlist->priv->popup);
 
                 gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3,
@@ -367,122 +373,13 @@ ario_songlist_popup_menu (ArioSonglist *songlist)
         }
 }
 
-static gboolean
-ario_songlist_button_press_cb (GtkWidget *widget,
-                               GdkEventButton *event,
-                               ArioSonglist *songlist)
+static void
+ario_songlist_activate_cb (ArioDndTree* tree,
+                           ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        GtkTreePath *path;
-        int x, y, bx, by;
-        gboolean selected;
-
-        if (!GTK_WIDGET_HAS_FOCUS (widget))
-                gtk_widget_grab_focus (widget);
-
-        if (songlist->priv->dragging)
-                return FALSE;
-
-        if (event->state & GDK_CONTROL_MASK || event->state & GDK_SHIFT_MASK)
-                return FALSE;
-
-        if (event->button == 1) {
-                gdk_window_get_pointer (widget->window, &x, &y, &mods);
-                gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget), x, y, &bx, &by);
-
-                if (bx >= 0 && by >= 0) {
-                        if (event->type == GDK_2BUTTON_PRESS) {
-                                ario_songlist_add_in_playlist (songlist, FALSE);
-                                return FALSE;
-                        }
-
-                        songlist->priv->drag_start_x = x;
-                        songlist->priv->drag_start_y = y;
-                        songlist->priv->pressed = TRUE;
-
-                        gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                        if (path) {
-                                selected = gtk_tree_selection_path_is_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), path);
-                                gtk_tree_path_free (path);
-
-                                return selected;
-                        }
-
-                        return TRUE;
-                } else {
-                        return FALSE;
-                }
-        }
-
-        if (event->button == 3) {
-                gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                if (path) {
-                        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-                        if (!gtk_tree_selection_path_is_selected (selection, path)) {
-                                gtk_tree_selection_unselect_all (selection);
-                                gtk_tree_selection_select_path (selection, path);
-                        }
-                        ario_songlist_popup_menu (songlist);
-                        gtk_tree_path_free (path);
-                        return TRUE;
-                }
-        }
-
-        return FALSE;
-}
-
-static gboolean
-ario_songlist_button_release_cb (GtkWidget *widget,
-                                 GdkEventButton *event,
-                                 ArioSonglist *songlist)
-{
-        ARIO_LOG_FUNCTION_START
-        if (!songlist->priv->dragging && !(event->state & GDK_CONTROL_MASK) && !(event->state & GDK_SHIFT_MASK)) {
-                int bx, by;
-                gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget), event->x, event->y, &bx, &by);
-                if (bx >= 0 && by >= 0) {
-                        GtkTreePath *path;
-
-                        gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL);
-                        if (path) {
-                                GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
-                                gtk_tree_selection_unselect_all (selection);
-                                gtk_tree_selection_select_path (selection, path);
-                                gtk_tree_path_free (path);
-                        }
-                }
-        }
-
-        songlist->priv->dragging = FALSE;
-        songlist->priv->pressed = FALSE;
-
-        return FALSE;
-}
-
-static gboolean
-ario_songlist_motion_notify_cb (GtkWidget *widget,
-                                GdkEventMotion *event,
-                                ArioSonglist *songlist)
-{
-        // desactivated to make the logs more readable
-        // ARIO_LOG_FUNCTION_START
-        GdkModifierType mods;
-        int x, y;
-        int dx, dy;
-
-        if ((songlist->priv->dragging) || !(songlist->priv->pressed))
-                return FALSE;
-
-        gdk_window_get_pointer (widget->window, &x, &y, &mods);
-
-        dx = x - songlist->priv->drag_start_x;
-        dy = y - songlist->priv->drag_start_y;
-
-        if ((ario_util_abs (dx) > DRAG_THRESHOLD) || (ario_util_abs (dy) > DRAG_THRESHOLD))
-                songlist->priv->dragging = TRUE;
-
-        return FALSE;
+        ARIO_LOG_FUNCTION_START;
+        /* Add selected songs to playlist */
+        ario_songlist_add_in_playlist (songlist, FALSE);
 }
 
 static void
@@ -491,12 +388,13 @@ ario_songlist_songlists_selection_drag_foreach (GtkTreeModel *model,
                                                 GtkTreeIter *iter,
                                                 gpointer userdata)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         GString *songlists = (GString *) userdata;
-        g_return_if_fail (songlists != NULL);
-
         gchar* val = NULL;
 
+        g_return_if_fail (songlists != NULL);
+
+        /* Append filename to dnd string */
         gtk_tree_model_get (model, iter, SONGS_FILENAME_COLUMN, &val, -1);
         g_string_append (songlists, val);
         g_string_append (songlists, "\n");
@@ -509,7 +407,7 @@ ario_songlist_drag_data_get_cb (GtkWidget * widget,
                                 GtkSelectionData * selection_data,
                                 guint info, guint time, gpointer data)
 {
-        ARIO_LOG_FUNCTION_START
+        ARIO_LOG_FUNCTION_START;
         ArioSonglist *songlist;
         GString* songlists = NULL;
 
@@ -519,11 +417,13 @@ ario_songlist_drag_data_get_cb (GtkWidget * widget,
         g_return_if_fail (widget != NULL);
         g_return_if_fail (selection_data != NULL);
 
+        /* Get dnd string with all selected songs */
         songlists = g_string_new("");
-        gtk_tree_selection_selected_foreach (songlist->priv->songlists_selection,
+        gtk_tree_selection_selected_foreach (songlist->priv->selection,
                                              ario_songlist_songlists_selection_drag_foreach,
                                              songlists);
 
+        /* Set drag data */
         gtk_selection_data_set (selection_data, selection_data->target, 8, (const guchar *) songlists->str,
                                 strlen (songlists->str) * sizeof(guchar));
 
@@ -533,6 +433,13 @@ ario_songlist_drag_data_get_cb (GtkWidget * widget,
 GtkListStore *
 ario_songlist_get_liststore (ArioSonglist *songlist)
 {
-        ARIO_LOG_FUNCTION_START
-        return songlist->priv->songlists_model;
+        ARIO_LOG_FUNCTION_START;
+        return songlist->priv->model;
+}
+
+GtkTreeSelection *
+ario_songlist_get_selection (ArioSonglist *songlist)
+{
+        ARIO_LOG_FUNCTION_START;
+        return songlist->priv->selection;
 }
